# -*- coding: iso-8859-1 -*-
"""Raw (C-style) API for OpenGL.GLUT

Automatically generated by the generateraw script, do not edit!
"""
from OpenGL.raw.GLUT.constants import *

from ctypes import *
from OpenGL import platform, arrays
from OpenGL.constant import Constant
from OpenGL import constants as GLconstants
GLvoid = GLconstants.GLvoid

FUNCTION_TYPE = platform.PLATFORM.functionTypeFor( platform.GLUT )

STRING = c_char_p
from OpenGL.constants import GLenum
from OpenGL.constants import GLfloat
from OpenGL.constants import GLdouble
from OpenGL.constants import GLint




# /usr/include/GL/freeglut_std.h 445
glutAddMenuEntry = platform.createBaseFunction( 
	'glutAddMenuEntry', dll=platform.GLUT, resultType=None, 
	argTypes=[STRING,c_int],
	doc='glutAddMenuEntry( STRING(label), c_int(value) ) -> None', 
	argNames=('label', 'value'),
)


# /usr/include/GL/freeglut_std.h 446
glutAddSubMenu = platform.createBaseFunction( 
	'glutAddSubMenu', dll=platform.GLUT, resultType=None, 
	argTypes=[STRING,c_int],
	doc='glutAddSubMenu( STRING(label), c_int(subMenu) ) -> None', 
	argNames=('label', 'subMenu'),
)


# /usr/include/GL/freeglut_std.h 450
glutAttachMenu = platform.createBaseFunction( 
	'glutAttachMenu', dll=platform.GLUT, resultType=None, 
	argTypes=[c_int],
	doc='glutAttachMenu( c_int(button) ) -> None', 
	argNames=('button',),
)


# /usr/include/GL/freeglut_std.h 499
glutBitmapCharacter = platform.createBaseFunction( 
	'glutBitmapCharacter', dll=platform.GLUT, resultType=None, 
	argTypes=[c_void_p,c_int],
	doc='glutBitmapCharacter( c_void_p(font), c_int(character) ) -> None', 
	argNames=('font', 'character'),
)


# /usr/include/GL/freeglut_std.h 503
glutBitmapLength = platform.createBaseFunction( 
	'glutBitmapLength', dll=platform.GLUT, resultType=c_int, 
	argTypes=[c_void_p,POINTER(c_ubyte)],
	doc='glutBitmapLength( c_void_p(font), POINTER(c_ubyte)(string) ) -> c_int', 
	argNames=('font', 'string'),
)


# /usr/include/GL/freeglut_std.h 500
glutBitmapWidth = platform.createBaseFunction( 
	'glutBitmapWidth', dll=platform.GLUT, resultType=c_int, 
	argTypes=[c_void_p,c_int],
	doc='glutBitmapWidth( c_void_p(font), c_int(character) ) -> c_int', 
	argNames=('font', 'character'),
)


# /usr/include/GL/freeglut_std.h 483
glutButtonBoxFunc = platform.createBaseFunction( 
	'glutButtonBoxFunc', dll=platform.GLUT, resultType=None, 
	argTypes=[FUNCTION_TYPE(None, c_int, c_int)],
	doc='glutButtonBoxFunc( FUNCTION_TYPE(None, c_int, c_int)(callback) ) -> None', 
	argNames=('callback',),
)


# /usr/include/GL/freeglut_std.h 447
glutChangeToMenuEntry = platform.createBaseFunction( 
	'glutChangeToMenuEntry', dll=platform.GLUT, resultType=None, 
	argTypes=[c_int,STRING,c_int],
	doc='glutChangeToMenuEntry( c_int(item), STRING(label), c_int(value) ) -> None', 
	argNames=('item', 'label', 'value'),
)


# /usr/include/GL/freeglut_std.h 448
glutChangeToSubMenu = platform.createBaseFunction( 
	'glutChangeToSubMenu', dll=platform.GLUT, resultType=None, 
	argTypes=[c_int,STRING,c_int],
	doc='glutChangeToSubMenu( c_int(item), STRING(label), c_int(value) ) -> None', 
	argNames=('item', 'label', 'value'),
)


# /usr/include/GL/freeglut_std.h 555
glutCopyColormap = platform.createBaseFunction( 
	'glutCopyColormap', dll=platform.GLUT, resultType=None, 
	argTypes=[c_int],
	doc='glutCopyColormap( c_int(window) ) -> None', 
	argNames=('window',),
)


# /usr/include/GL/freeglut_std.h 441
glutCreateMenu = platform.createBaseFunction( 
	'glutCreateMenu', dll=platform.GLUT, resultType=c_int, 
	argTypes=[FUNCTION_TYPE(c_int, c_int)],
	doc='glutCreateMenu( FUNCTION_TYPE(c_int, c_int)(callback) ) -> c_int', 
	argNames=('callback',),
)


# /usr/include/GL/freeglut_std.h 399
glutCreateSubWindow = platform.createBaseFunction( 
	'glutCreateSubWindow', dll=platform.GLUT, resultType=c_int, 
	argTypes=[c_int,c_int,c_int,c_int,c_int],
	doc='glutCreateSubWindow( c_int(window), c_int(x), c_int(y), c_int(width), c_int(height) ) -> c_int', 
	argNames=('window', 'x', 'y', 'width', 'height'),
)


# /usr/include/GL/freeglut_std.h 398
glutCreateWindow = platform.createBaseFunction( 
	'glutCreateWindow', dll=platform.GLUT, resultType=c_int, 
	argTypes=[STRING],
	doc='glutCreateWindow( STRING(title) ) -> c_int', 
	argNames=('title',),
)


# /usr/include/GL/freeglut_std.h 442
glutDestroyMenu = platform.createBaseFunction( 
	'glutDestroyMenu', dll=platform.GLUT, resultType=None, 
	argTypes=[c_int],
	doc='glutDestroyMenu( c_int(menu) ) -> None', 
	argNames=('menu',),
)


# /usr/include/GL/freeglut_std.h 400
glutDestroyWindow = platform.createBaseFunction( 
	'glutDestroyWindow', dll=platform.GLUT, resultType=None, 
	argTypes=[c_int],
	doc='glutDestroyWindow( c_int(window) ) -> None', 
	argNames=('window',),
)


# /usr/include/GL/freeglut_std.h 451
glutDetachMenu = platform.createBaseFunction( 
	'glutDetachMenu', dll=platform.GLUT, resultType=None, 
	argTypes=[c_int],
	doc='glutDetachMenu( c_int(button) ) -> None', 
	argNames=('button',),
)


# /usr/include/GL/freeglut_std.h 492
glutDeviceGet = platform.createBaseFunction( 
	'glutDeviceGet', dll=platform.GLUT, resultType=c_int, 
	argTypes=[GLenum],
	doc='glutDeviceGet( GLenum(query) ) -> c_int', 
	argNames=('query',),
)


# /usr/include/GL/freeglut_std.h 484
glutDialsFunc = platform.createBaseFunction( 
	'glutDialsFunc', dll=platform.GLUT, resultType=None, 
	argTypes=[FUNCTION_TYPE(None, c_int, c_int)],
	doc='glutDialsFunc( FUNCTION_TYPE(None, c_int, c_int)(callback) ) -> None', 
	argNames=('callback',),
)


# /usr/include/GL/freeglut_std.h 466
glutDisplayFunc = platform.createBaseFunction( 
	'glutDisplayFunc', dll=platform.GLUT, resultType=None, 
	argTypes=[FUNCTION_TYPE(None)],
	doc='glutDisplayFunc( FUNCTION_TYPE(None)(callback) ) -> None', 
	argNames=('callback',),
)


# /usr/include/GL/freeglut_std.h 537
glutEnterGameMode = platform.createBaseFunction( 
	'glutEnterGameMode', dll=platform.GLUT, resultType=c_int, 
	argTypes=[],
	doc='glutEnterGameMode(  ) -> c_int', 
	argNames=(),
)


# /usr/include/GL/freeglut_std.h 470
glutEntryFunc = platform.createBaseFunction( 
	'glutEntryFunc', dll=platform.GLUT, resultType=None, 
	argTypes=[FUNCTION_TYPE(None, c_int)],
	doc='glutEntryFunc( FUNCTION_TYPE(None, c_int)(callback) ) -> None', 
	argNames=('callback',),
)


# /usr/include/GL/freeglut_std.h 430
glutEstablishOverlay = platform.createBaseFunction( 
	'glutEstablishOverlay', dll=platform.GLUT, resultType=None, 
	argTypes=[],
	doc='glutEstablishOverlay(  ) -> None', 
	argNames=(),
)


# /usr/include/GL/freeglut_std.h 567
glutExtensionSupported = platform.createBaseFunction( 
	'glutExtensionSupported', dll=platform.GLUT, resultType=c_int, 
	argTypes=[STRING],
	doc='glutExtensionSupported( STRING(extension) ) -> c_int', 
	argNames=('extension',),
)


# /usr/include/GL/freeglut_std.h 562
glutForceJoystickFunc = platform.createBaseFunction( 
	'glutForceJoystickFunc', dll=platform.GLUT, resultType=None, 
	argTypes=[],
	doc='glutForceJoystickFunc(  ) -> None', 
	argNames=(),
)


# /usr/include/GL/freeglut_std.h 412
glutFullScreen = platform.createBaseFunction( 
	'glutFullScreen', dll=platform.GLUT, resultType=None, 
	argTypes=[],
	doc='glutFullScreen(  ) -> None', 
	argNames=(),
)


# /usr/include/GL/freeglut_std.h 539
glutGameModeGet = platform.createBaseFunction( 
	'glutGameModeGet', dll=platform.GLUT, resultType=c_int, 
	argTypes=[GLenum],
	doc='glutGameModeGet( GLenum(query) ) -> c_int', 
	argNames=('query',),
)


# /usr/include/GL/freeglut_std.h 536
glutGameModeString = platform.createBaseFunction( 
	'glutGameModeString', dll=platform.GLUT, resultType=None, 
	argTypes=[STRING],
	doc='glutGameModeString( STRING(string) ) -> None', 
	argNames=('string',),
)


# /usr/include/GL/freeglut_std.h 491
glutGet = platform.createBaseFunction( 
	'glutGet', dll=platform.GLUT, resultType=c_int, 
	argTypes=[GLenum],
	doc='glutGet( GLenum(query) ) -> c_int', 
	argNames=('query',),
)


# /usr/include/GL/freeglut_std.h 554
glutGetColor = platform.createBaseFunction( 
	'glutGetColor', dll=platform.GLUT, resultType=GLfloat, 
	argTypes=[c_int,c_int],
	doc='glutGetColor( c_int(color), c_int(component) ) -> GLfloat', 
	argNames=('color', 'component'),
)


# /usr/include/GL/freeglut_std.h 443
glutGetMenu = platform.createBaseFunction( 
	'glutGetMenu', dll=platform.GLUT, resultType=c_int, 
	argTypes=[],
	doc='glutGetMenu(  ) -> c_int', 
	argNames=(),
)


# /usr/include/GL/freeglut_std.h 493
glutGetModifiers = platform.createBaseFunction( 
	'glutGetModifiers', dll=platform.GLUT, resultType=c_int, 
	argTypes=[],
	doc='glutGetModifiers(  ) -> c_int', 
	argNames=(),
)


# /usr/include/GL/freeglut_std.h 402
glutGetWindow = platform.createBaseFunction( 
	'glutGetWindow', dll=platform.GLUT, resultType=c_int, 
	argTypes=[],
	doc='glutGetWindow(  ) -> c_int', 
	argNames=(),
)


# /usr/include/GL/freeglut_std.h 436
glutHideOverlay = platform.createBaseFunction( 
	'glutHideOverlay', dll=platform.GLUT, resultType=None, 
	argTypes=[],
	doc='glutHideOverlay(  ) -> None', 
	argNames=(),
)


# /usr/include/GL/freeglut_std.h 408
glutHideWindow = platform.createBaseFunction( 
	'glutHideWindow', dll=platform.GLUT, resultType=None, 
	argTypes=[],
	doc='glutHideWindow(  ) -> None', 
	argNames=(),
)


# /usr/include/GL/freeglut_std.h 409
glutIconifyWindow = platform.createBaseFunction( 
	'glutIconifyWindow', dll=platform.GLUT, resultType=None, 
	argTypes=[],
	doc='glutIconifyWindow(  ) -> None', 
	argNames=(),
)


# /usr/include/GL/freeglut_std.h 457
glutIdleFunc = platform.createBaseFunction( 
	'glutIdleFunc', dll=platform.GLUT, resultType=None, 
	argTypes=[FUNCTION_TYPE(None)],
	doc='glutIdleFunc( FUNCTION_TYPE(None)(callback) ) -> None', 
	argNames=('callback',),
)


# /usr/include/GL/freeglut_std.h 560
glutIgnoreKeyRepeat = platform.createBaseFunction( 
	'glutIgnoreKeyRepeat', dll=platform.GLUT, resultType=None, 
	argTypes=[c_int],
	doc='glutIgnoreKeyRepeat( c_int(ignore) ) -> None', 
	argNames=('ignore',),
)


# /usr/include/GL/freeglut_std.h 384
glutInit = platform.createBaseFunction( 
	'glutInit', dll=platform.GLUT, resultType=None, 
	argTypes=[POINTER(c_int),POINTER(STRING)],
	doc='glutInit( POINTER(c_int)(pargc), POINTER(STRING)(argv) ) -> None', 
	argNames=('pargc', 'argv'),
)


# /usr/include/GL/freeglut_std.h 387
glutInitDisplayMode = platform.createBaseFunction( 
	'glutInitDisplayMode', dll=platform.GLUT, resultType=None, 
	argTypes=[c_uint],
	doc='glutInitDisplayMode( c_uint(displayMode) ) -> None', 
	argNames=('displayMode',),
)


# /usr/include/GL/freeglut_std.h 388
glutInitDisplayString = platform.createBaseFunction( 
	'glutInitDisplayString', dll=platform.GLUT, resultType=None, 
	argTypes=[STRING],
	doc='glutInitDisplayString( STRING(displayMode) ) -> None', 
	argNames=('displayMode',),
)


# /usr/include/GL/freeglut_std.h 385
glutInitWindowPosition = platform.createBaseFunction( 
	'glutInitWindowPosition', dll=platform.GLUT, resultType=None, 
	argTypes=[c_int,c_int],
	doc='glutInitWindowPosition( c_int(x), c_int(y) ) -> None', 
	argNames=('x', 'y'),
)


# /usr/include/GL/freeglut_std.h 386
glutInitWindowSize = platform.createBaseFunction( 
	'glutInitWindowSize', dll=platform.GLUT, resultType=None, 
	argTypes=[c_int,c_int],
	doc='glutInitWindowSize( c_int(width), c_int(height) ) -> None', 
	argNames=('width', 'height'),
)


# /usr/include/GL/freeglut_std.h 474
glutJoystickFunc = platform.createBaseFunction( 
	'glutJoystickFunc', dll=platform.GLUT, resultType=None, 
	argTypes=[FUNCTION_TYPE(None, c_uint, c_int, c_int, c_int),c_int],
	doc='glutJoystickFunc( FUNCTION_TYPE(None, c_uint, c_int, c_int, c_int)(callback), c_int(pollInterval) ) -> None', 
	argNames=('callback', 'pollInterval'),
)


# /usr/include/GL/freeglut_std.h 462
glutKeyboardFunc = platform.createBaseFunction( 
	'glutKeyboardFunc', dll=platform.GLUT, resultType=None, 
	argTypes=[FUNCTION_TYPE(None, c_ubyte, c_int, c_int)],
	doc='glutKeyboardFunc( FUNCTION_TYPE(None, c_ubyte, c_int, c_int)(callback) ) -> None', 
	argNames=('callback',),
)


# /usr/include/GL/freeglut_std.h 472
glutKeyboardUpFunc = platform.createBaseFunction( 
	'glutKeyboardUpFunc', dll=platform.GLUT, resultType=None, 
	argTypes=[FUNCTION_TYPE(None, c_ubyte, c_int, c_int)],
	doc='glutKeyboardUpFunc( FUNCTION_TYPE(None, c_ubyte, c_int, c_int)(callback) ) -> None', 
	argNames=('callback',),
)


# /usr/include/GL/freeglut_std.h 494
glutLayerGet = platform.createBaseFunction( 
	'glutLayerGet', dll=platform.GLUT, resultType=c_int, 
	argTypes=[GLenum],
	doc='glutLayerGet( GLenum(query) ) -> c_int', 
	argNames=('query',),
)


# /usr/include/GL/freeglut_std.h 538
glutLeaveGameMode = platform.createBaseFunction( 
	'glutLeaveGameMode', dll=platform.GLUT, resultType=None, 
	argTypes=[],
	doc='glutLeaveGameMode(  ) -> None', 
	argNames=(),
)


# /usr/include/GL/freeglut_std.h 393
glutMainLoop = platform.createBaseFunction( 
	'glutMainLoop', dll=platform.GLUT, resultType=None, 
	argTypes=[],
	doc='glutMainLoop(  ) -> None', 
	argNames=(),
)


# /usr/include/GL/freeglut_std.h 475
glutMenuStateFunc = platform.createBaseFunction( 
	'glutMenuStateFunc', dll=platform.GLUT, resultType=None, 
	argTypes=[FUNCTION_TYPE(None, c_int)],
	doc='glutMenuStateFunc( FUNCTION_TYPE(None, c_int)(callback) ) -> None', 
	argNames=('callback',),
)


# /usr/include/GL/freeglut_std.h 476
glutMenuStatusFunc = platform.createBaseFunction( 
	'glutMenuStatusFunc', dll=platform.GLUT, resultType=None, 
	argTypes=[FUNCTION_TYPE(None, c_int, c_int, c_int)],
	doc='glutMenuStatusFunc( FUNCTION_TYPE(None, c_int, c_int, c_int)(callback) ) -> None', 
	argNames=('callback',),
)


# /usr/include/GL/freeglut_std.h 468
glutMotionFunc = platform.createBaseFunction( 
	'glutMotionFunc', dll=platform.GLUT, resultType=None, 
	argTypes=[FUNCTION_TYPE(None, c_int, c_int)],
	doc='glutMotionFunc( FUNCTION_TYPE(None, c_int, c_int)(callback) ) -> None', 
	argNames=('callback',),
)


# /usr/include/GL/freeglut_std.h 467
glutMouseFunc = platform.createBaseFunction( 
	'glutMouseFunc', dll=platform.GLUT, resultType=None, 
	argTypes=[FUNCTION_TYPE(None, c_int, c_int, c_int, c_int)],
	doc='glutMouseFunc( FUNCTION_TYPE(None, c_int, c_int, c_int, c_int)(callback) ) -> None', 
	argNames=('callback',),
)


# /usr/include/GL/freeglut_std.h 477
glutOverlayDisplayFunc = platform.createBaseFunction( 
	'glutOverlayDisplayFunc', dll=platform.GLUT, resultType=None, 
	argTypes=[FUNCTION_TYPE(None)],
	doc='glutOverlayDisplayFunc( FUNCTION_TYPE(None)(callback) ) -> None', 
	argNames=('callback',),
)


# /usr/include/GL/freeglut_std.h 469
glutPassiveMotionFunc = platform.createBaseFunction( 
	'glutPassiveMotionFunc', dll=platform.GLUT, resultType=None, 
	argTypes=[FUNCTION_TYPE(None, c_int, c_int)],
	doc='glutPassiveMotionFunc( FUNCTION_TYPE(None, c_int, c_int)(callback) ) -> None', 
	argNames=('callback',),
)


# /usr/include/GL/freeglut_std.h 411
glutPopWindow = platform.createBaseFunction( 
	'glutPopWindow', dll=platform.GLUT, resultType=None, 
	argTypes=[],
	doc='glutPopWindow(  ) -> None', 
	argNames=(),
)


# /usr/include/GL/freeglut_std.h 406
glutPositionWindow = platform.createBaseFunction( 
	'glutPositionWindow', dll=platform.GLUT, resultType=None, 
	argTypes=[c_int,c_int],
	doc='glutPositionWindow( c_int(x), c_int(y) ) -> None', 
	argNames=('x', 'y'),
)


# /usr/include/GL/freeglut_std.h 433
glutPostOverlayRedisplay = platform.createBaseFunction( 
	'glutPostOverlayRedisplay', dll=platform.GLUT, resultType=None, 
	argTypes=[],
	doc='glutPostOverlayRedisplay(  ) -> None', 
	argNames=(),
)


# /usr/include/GL/freeglut_std.h 418
glutPostRedisplay = platform.createBaseFunction( 
	'glutPostRedisplay', dll=platform.GLUT, resultType=None, 
	argTypes=[],
	doc='glutPostRedisplay(  ) -> None', 
	argNames=(),
)


# /usr/include/GL/freeglut_std.h 434
glutPostWindowOverlayRedisplay = platform.createBaseFunction( 
	'glutPostWindowOverlayRedisplay', dll=platform.GLUT, resultType=None, 
	argTypes=[c_int],
	doc='glutPostWindowOverlayRedisplay( c_int(window) ) -> None', 
	argNames=('window',),
)


# /usr/include/GL/freeglut_std.h 417
glutPostWindowRedisplay = platform.createBaseFunction( 
	'glutPostWindowRedisplay', dll=platform.GLUT, resultType=None, 
	argTypes=[c_int],
	doc='glutPostWindowRedisplay( c_int(window) ) -> None', 
	argNames=('window',),
)


# /usr/include/GL/freeglut_std.h 410
glutPushWindow = platform.createBaseFunction( 
	'glutPushWindow', dll=platform.GLUT, resultType=None, 
	argTypes=[],
	doc='glutPushWindow(  ) -> None', 
	argNames=(),
)


# /usr/include/GL/freeglut_std.h 449
glutRemoveMenuItem = platform.createBaseFunction( 
	'glutRemoveMenuItem', dll=platform.GLUT, resultType=None, 
	argTypes=[c_int],
	doc='glutRemoveMenuItem( c_int(item) ) -> None', 
	argNames=('item',),
)


# /usr/include/GL/freeglut_std.h 431
glutRemoveOverlay = platform.createBaseFunction( 
	'glutRemoveOverlay', dll=platform.GLUT, resultType=None, 
	argTypes=[],
	doc='glutRemoveOverlay(  ) -> None', 
	argNames=(),
)


# /usr/include/GL/freeglut_std.h 568
glutReportErrors = platform.createBaseFunction( 
	'glutReportErrors', dll=platform.GLUT, resultType=None, 
	argTypes=[],
	doc='glutReportErrors(  ) -> None', 
	argNames=(),
)


# /usr/include/GL/freeglut_std.h 464
glutReshapeFunc = platform.createBaseFunction( 
	'glutReshapeFunc', dll=platform.GLUT, resultType=None, 
	argTypes=[FUNCTION_TYPE(None, c_int, c_int)],
	doc='glutReshapeFunc( FUNCTION_TYPE(None, c_int, c_int)(callback) ) -> None', 
	argNames=('callback',),
)


# /usr/include/GL/freeglut_std.h 405
glutReshapeWindow = platform.createBaseFunction( 
	'glutReshapeWindow', dll=platform.GLUT, resultType=None, 
	argTypes=[c_int,c_int],
	doc='glutReshapeWindow( c_int(width), c_int(height) ) -> None', 
	argNames=('width', 'height'),
)


# /usr/include/GL/freeglut_std.h 553
glutSetColor = platform.createBaseFunction( 
	'glutSetColor', dll=platform.GLUT, resultType=None, 
	argTypes=[c_int,GLfloat,GLfloat,GLfloat],
	doc='glutSetColor( c_int(color), GLfloat(red), GLfloat(green), GLfloat(blue) ) -> None', 
	argNames=('color', 'red', 'green', 'blue'),
)


# /usr/include/GL/freeglut_std.h 425
glutSetCursor = platform.createBaseFunction( 
	'glutSetCursor', dll=platform.GLUT, resultType=None, 
	argTypes=[c_int],
	doc='glutSetCursor( c_int(cursor) ) -> None', 
	argNames=('cursor',),
)


# /usr/include/GL/freeglut_std.h 404
glutSetIconTitle = platform.createBaseFunction( 
	'glutSetIconTitle', dll=platform.GLUT, resultType=None, 
	argTypes=[STRING],
	doc='glutSetIconTitle( STRING(title) ) -> None', 
	argNames=('title',),
)


# /usr/include/GL/freeglut_std.h 561
glutSetKeyRepeat = platform.createBaseFunction( 
	'glutSetKeyRepeat', dll=platform.GLUT, resultType=None, 
	argTypes=[c_int],
	doc='glutSetKeyRepeat( c_int(repeatMode) ) -> None', 
	argNames=('repeatMode',),
)


# /usr/include/GL/freeglut_std.h 444
glutSetMenu = platform.createBaseFunction( 
	'glutSetMenu', dll=platform.GLUT, resultType=None, 
	argTypes=[c_int],
	doc='glutSetMenu( c_int(menu) ) -> None', 
	argNames=('menu',),
)


# /usr/include/GL/freeglut_std.h 401
glutSetWindow = platform.createBaseFunction( 
	'glutSetWindow', dll=platform.GLUT, resultType=None, 
	argTypes=[c_int],
	doc='glutSetWindow( c_int(window) ) -> None', 
	argNames=('window',),
)


# /usr/include/GL/freeglut_std.h 403
glutSetWindowTitle = platform.createBaseFunction( 
	'glutSetWindowTitle', dll=platform.GLUT, resultType=None, 
	argTypes=[STRING],
	doc='glutSetWindowTitle( STRING(title) ) -> None', 
	argNames=('title',),
)


# /usr/include/GL/freeglut_std.h 545
glutSetupVideoResizing = platform.createBaseFunction( 
	'glutSetupVideoResizing', dll=platform.GLUT, resultType=None, 
	argTypes=[],
	doc='glutSetupVideoResizing(  ) -> None', 
	argNames=(),
)


# /usr/include/GL/freeglut_std.h 435
glutShowOverlay = platform.createBaseFunction( 
	'glutShowOverlay', dll=platform.GLUT, resultType=None, 
	argTypes=[],
	doc='glutShowOverlay(  ) -> None', 
	argNames=(),
)


# /usr/include/GL/freeglut_std.h 407
glutShowWindow = platform.createBaseFunction( 
	'glutShowWindow', dll=platform.GLUT, resultType=None, 
	argTypes=[],
	doc='glutShowWindow(  ) -> None', 
	argNames=(),
)


# /usr/include/GL/freeglut_std.h 514
glutSolidCone = platform.createBaseFunction( 
	'glutSolidCone', dll=platform.GLUT, resultType=None, 
	argTypes=[GLdouble,GLdouble,GLint,GLint],
	doc='glutSolidCone( GLdouble(base), GLdouble(height), GLint(slices), GLint(stacks) ) -> None', 
	argNames=('base', 'height', 'slices', 'stacks'),
)


# /usr/include/GL/freeglut_std.h 510
glutSolidCube = platform.createBaseFunction( 
	'glutSolidCube', dll=platform.GLUT, resultType=None, 
	argTypes=[GLdouble],
	doc='glutSolidCube( GLdouble(size) ) -> None', 
	argNames=('size',),
)


# /usr/include/GL/freeglut_std.h 519
glutSolidDodecahedron = platform.createBaseFunction( 
	'glutSolidDodecahedron', dll=platform.GLUT, resultType=None, 
	argTypes=[],
	doc='glutSolidDodecahedron(  ) -> None', 
	argNames=(),
)


# /usr/include/GL/freeglut_std.h 525
glutSolidIcosahedron = platform.createBaseFunction( 
	'glutSolidIcosahedron', dll=platform.GLUT, resultType=None, 
	argTypes=[],
	doc='glutSolidIcosahedron(  ) -> None', 
	argNames=(),
)


# /usr/include/GL/freeglut_std.h 521
glutSolidOctahedron = platform.createBaseFunction( 
	'glutSolidOctahedron', dll=platform.GLUT, resultType=None, 
	argTypes=[],
	doc='glutSolidOctahedron(  ) -> None', 
	argNames=(),
)


# /usr/include/GL/freeglut_std.h 512
glutSolidSphere = platform.createBaseFunction( 
	'glutSolidSphere', dll=platform.GLUT, resultType=None, 
	argTypes=[GLdouble,GLint,GLint],
	doc='glutSolidSphere( GLdouble(radius), GLint(slices), GLint(stacks) ) -> None', 
	argNames=('radius', 'slices', 'stacks'),
)


# /usr/include/GL/freeglut_std.h 531
glutSolidTeapot = platform.createBaseFunction( 
	'glutSolidTeapot', dll=platform.GLUT, resultType=None, 
	argTypes=[GLdouble],
	doc='glutSolidTeapot( GLdouble(size) ) -> None', 
	argNames=('size',),
)


# /usr/include/GL/freeglut_std.h 523
glutSolidTetrahedron = platform.createBaseFunction( 
	'glutSolidTetrahedron', dll=platform.GLUT, resultType=None, 
	argTypes=[],
	doc='glutSolidTetrahedron(  ) -> None', 
	argNames=(),
)


# /usr/include/GL/freeglut_std.h 517
glutSolidTorus = platform.createBaseFunction( 
	'glutSolidTorus', dll=platform.GLUT, resultType=None, 
	argTypes=[GLdouble,GLdouble,GLint,GLint],
	doc='glutSolidTorus( GLdouble(innerRadius), GLdouble(outerRadius), GLint(sides), GLint(rings) ) -> None', 
	argNames=('innerRadius', 'outerRadius', 'sides', 'rings'),
)


# /usr/include/GL/freeglut_std.h 482
glutSpaceballButtonFunc = platform.createBaseFunction( 
	'glutSpaceballButtonFunc', dll=platform.GLUT, resultType=None, 
	argTypes=[FUNCTION_TYPE(None, c_int, c_int)],
	doc='glutSpaceballButtonFunc( FUNCTION_TYPE(None, c_int, c_int)(callback) ) -> None', 
	argNames=('callback',),
)


# /usr/include/GL/freeglut_std.h 480
glutSpaceballMotionFunc = platform.createBaseFunction( 
	'glutSpaceballMotionFunc', dll=platform.GLUT, resultType=None, 
	argTypes=[FUNCTION_TYPE(None, c_int, c_int, c_int)],
	doc='glutSpaceballMotionFunc( FUNCTION_TYPE(None, c_int, c_int, c_int)(callback) ) -> None', 
	argNames=('callback',),
)


# /usr/include/GL/freeglut_std.h 481
glutSpaceballRotateFunc = platform.createBaseFunction( 
	'glutSpaceballRotateFunc', dll=platform.GLUT, resultType=None, 
	argTypes=[FUNCTION_TYPE(None, c_int, c_int, c_int)],
	doc='glutSpaceballRotateFunc( FUNCTION_TYPE(None, c_int, c_int, c_int)(callback) ) -> None', 
	argNames=('callback',),
)


# /usr/include/GL/freeglut_std.h 463
glutSpecialFunc = platform.createBaseFunction( 
	'glutSpecialFunc', dll=platform.GLUT, resultType=None, 
	argTypes=[FUNCTION_TYPE(None, c_int, c_int, c_int)],
	doc='glutSpecialFunc( FUNCTION_TYPE(None, c_int, c_int, c_int)(callback) ) -> None', 
	argNames=('callback',),
)


# /usr/include/GL/freeglut_std.h 473
glutSpecialUpFunc = platform.createBaseFunction( 
	'glutSpecialUpFunc', dll=platform.GLUT, resultType=None, 
	argTypes=[FUNCTION_TYPE(None, c_int, c_int, c_int)],
	doc='glutSpecialUpFunc( FUNCTION_TYPE(None, c_int, c_int, c_int)(callback) ) -> None', 
	argNames=('callback',),
)


# /usr/include/GL/freeglut_std.h 546
glutStopVideoResizing = platform.createBaseFunction( 
	'glutStopVideoResizing', dll=platform.GLUT, resultType=None, 
	argTypes=[],
	doc='glutStopVideoResizing(  ) -> None', 
	argNames=(),
)


# /usr/include/GL/freeglut_std.h 501
glutStrokeCharacter = platform.createBaseFunction( 
	'glutStrokeCharacter', dll=platform.GLUT, resultType=None, 
	argTypes=[c_void_p,c_int],
	doc='glutStrokeCharacter( c_void_p(font), c_int(character) ) -> None', 
	argNames=('font', 'character'),
)


# /usr/include/GL/freeglut_std.h 504
glutStrokeLength = platform.createBaseFunction( 
	'glutStrokeLength', dll=platform.GLUT, resultType=c_int, 
	argTypes=[c_void_p,POINTER(c_ubyte)],
	doc='glutStrokeLength( c_void_p(font), POINTER(c_ubyte)(string) ) -> c_int', 
	argNames=('font', 'string'),
)


# /usr/include/GL/freeglut_std.h 502
glutStrokeWidth = platform.createBaseFunction( 
	'glutStrokeWidth', dll=platform.GLUT, resultType=c_int, 
	argTypes=[c_void_p,c_int],
	doc='glutStrokeWidth( c_void_p(font), c_int(character) ) -> c_int', 
	argNames=('font', 'character'),
)


# /usr/include/GL/freeglut_std.h 419
glutSwapBuffers = platform.createBaseFunction( 
	'glutSwapBuffers', dll=platform.GLUT, resultType=None, 
	argTypes=[],
	doc='glutSwapBuffers(  ) -> None', 
	argNames=(),
)


# /usr/include/GL/freeglut_std.h 486
glutTabletButtonFunc = platform.createBaseFunction( 
	'glutTabletButtonFunc', dll=platform.GLUT, resultType=None, 
	argTypes=[FUNCTION_TYPE(None, c_int, c_int, c_int, c_int)],
	doc='glutTabletButtonFunc( FUNCTION_TYPE(None, c_int, c_int, c_int, c_int)(callback) ) -> None', 
	argNames=('callback',),
)


# /usr/include/GL/freeglut_std.h 485
glutTabletMotionFunc = platform.createBaseFunction( 
	'glutTabletMotionFunc', dll=platform.GLUT, resultType=None, 
	argTypes=[FUNCTION_TYPE(None, c_int, c_int)],
	doc='glutTabletMotionFunc( FUNCTION_TYPE(None, c_int, c_int)(callback) ) -> None', 
	argNames=('callback',),
)


# /usr/include/GL/freeglut_std.h 456
glutTimerFunc = platform.createBaseFunction( 
	'glutTimerFunc', dll=platform.GLUT, resultType=None, 
	argTypes=[c_uint,FUNCTION_TYPE(None, c_int),c_int],
	doc='glutTimerFunc( c_uint(time), FUNCTION_TYPE(None, c_int)(callback), c_int(value) ) -> None', 
	argNames=('time', 'callback', 'value'),
)


# /usr/include/GL/freeglut_std.h 432
glutUseLayer = platform.createBaseFunction( 
	'glutUseLayer', dll=platform.GLUT, resultType=None, 
	argTypes=[GLenum],
	doc='glutUseLayer( GLenum(layer) ) -> None', 
	argNames=('layer',),
)


# /usr/include/GL/freeglut_std.h 548
glutVideoPan = platform.createBaseFunction( 
	'glutVideoPan', dll=platform.GLUT, resultType=None, 
	argTypes=[c_int,c_int,c_int,c_int],
	doc='glutVideoPan( c_int(x), c_int(y), c_int(width), c_int(height) ) -> None', 
	argNames=('x', 'y', 'width', 'height'),
)


# /usr/include/GL/freeglut_std.h 547
glutVideoResize = platform.createBaseFunction( 
	'glutVideoResize', dll=platform.GLUT, resultType=None, 
	argTypes=[c_int,c_int,c_int,c_int],
	doc='glutVideoResize( c_int(x), c_int(y), c_int(width), c_int(height) ) -> None', 
	argNames=('x', 'y', 'width', 'height'),
)


# /usr/include/GL/freeglut_std.h 544
glutVideoResizeGet = platform.createBaseFunction( 
	'glutVideoResizeGet', dll=platform.GLUT, resultType=c_int, 
	argTypes=[GLenum],
	doc='glutVideoResizeGet( GLenum(query) ) -> c_int', 
	argNames=('query',),
)


# /usr/include/GL/freeglut_std.h 465
glutVisibilityFunc = platform.createBaseFunction( 
	'glutVisibilityFunc', dll=platform.GLUT, resultType=None, 
	argTypes=[FUNCTION_TYPE(None, c_int)],
	doc='glutVisibilityFunc( FUNCTION_TYPE(None, c_int)(callback) ) -> None', 
	argNames=('callback',),
)


# /usr/include/GL/freeglut_std.h 424
glutWarpPointer = platform.createBaseFunction( 
	'glutWarpPointer', dll=platform.GLUT, resultType=None, 
	argTypes=[c_int,c_int],
	doc='glutWarpPointer( c_int(x), c_int(y) ) -> None', 
	argNames=('x', 'y'),
)


# /usr/include/GL/freeglut_std.h 478
glutWindowStatusFunc = platform.createBaseFunction( 
	'glutWindowStatusFunc', dll=platform.GLUT, resultType=None, 
	argTypes=[FUNCTION_TYPE(None, c_int)],
	doc='glutWindowStatusFunc( FUNCTION_TYPE(None, c_int)(callback) ) -> None', 
	argNames=('callback',),
)


# /usr/include/GL/freeglut_std.h 513
glutWireCone = platform.createBaseFunction( 
	'glutWireCone', dll=platform.GLUT, resultType=None, 
	argTypes=[GLdouble,GLdouble,GLint,GLint],
	doc='glutWireCone( GLdouble(base), GLdouble(height), GLint(slices), GLint(stacks) ) -> None', 
	argNames=('base', 'height', 'slices', 'stacks'),
)


# /usr/include/GL/freeglut_std.h 509
glutWireCube = platform.createBaseFunction( 
	'glutWireCube', dll=platform.GLUT, resultType=None, 
	argTypes=[GLdouble],
	doc='glutWireCube( GLdouble(size) ) -> None', 
	argNames=('size',),
)


# /usr/include/GL/freeglut_std.h 518
glutWireDodecahedron = platform.createBaseFunction( 
	'glutWireDodecahedron', dll=platform.GLUT, resultType=None, 
	argTypes=[],
	doc='glutWireDodecahedron(  ) -> None', 
	argNames=(),
)


# /usr/include/GL/freeglut_std.h 524
glutWireIcosahedron = platform.createBaseFunction( 
	'glutWireIcosahedron', dll=platform.GLUT, resultType=None, 
	argTypes=[],
	doc='glutWireIcosahedron(  ) -> None', 
	argNames=(),
)


# /usr/include/GL/freeglut_std.h 520
glutWireOctahedron = platform.createBaseFunction( 
	'glutWireOctahedron', dll=platform.GLUT, resultType=None, 
	argTypes=[],
	doc='glutWireOctahedron(  ) -> None', 
	argNames=(),
)


# /usr/include/GL/freeglut_std.h 511
glutWireSphere = platform.createBaseFunction( 
	'glutWireSphere', dll=platform.GLUT, resultType=None, 
	argTypes=[GLdouble,GLint,GLint],
	doc='glutWireSphere( GLdouble(radius), GLint(slices), GLint(stacks) ) -> None', 
	argNames=('radius', 'slices', 'stacks'),
)


# /usr/include/GL/freeglut_std.h 530
glutWireTeapot = platform.createBaseFunction( 
	'glutWireTeapot', dll=platform.GLUT, resultType=None, 
	argTypes=[GLdouble],
	doc='glutWireTeapot( GLdouble(size) ) -> None', 
	argNames=('size',),
)


# /usr/include/GL/freeglut_std.h 522
glutWireTetrahedron = platform.createBaseFunction( 
	'glutWireTetrahedron', dll=platform.GLUT, resultType=None, 
	argTypes=[],
	doc='glutWireTetrahedron(  ) -> None', 
	argNames=(),
)


# /usr/include/GL/freeglut_std.h 516
glutWireTorus = platform.createBaseFunction( 
	'glutWireTorus', dll=platform.GLUT, resultType=None, 
	argTypes=[GLdouble,GLdouble,GLint,GLint],
	doc='glutWireTorus( GLdouble(innerRadius), GLdouble(outerRadius), GLint(sides), GLint(rings) ) -> None', 
	argNames=('innerRadius', 'outerRadius', 'sides', 'rings'),
)

__all__ = [
	'GLUT_ACCUM',
	'GLUT_ACTIVE_ALT',
	'GLUT_ACTIVE_CTRL',
	'GLUT_ACTIVE_SHIFT',
	'GLUT_ALPHA',
	'GLUT_API_VERSION',
	'GLUT_BLUE',
	'GLUT_CURSOR_BOTTOM_LEFT_CORNER',
	'GLUT_CURSOR_BOTTOM_RIGHT_CORNER',
	'GLUT_CURSOR_BOTTOM_SIDE',
	'GLUT_CURSOR_CROSSHAIR',
	'GLUT_CURSOR_CYCLE',
	'GLUT_CURSOR_DESTROY',
	'GLUT_CURSOR_FULL_CROSSHAIR',
	'GLUT_CURSOR_HELP',
	'GLUT_CURSOR_INFO',
	'GLUT_CURSOR_INHERIT',
	'GLUT_CURSOR_LEFT_ARROW',
	'GLUT_CURSOR_LEFT_RIGHT',
	'GLUT_CURSOR_LEFT_SIDE',
	'GLUT_CURSOR_NONE',
	'GLUT_CURSOR_RIGHT_ARROW',
	'GLUT_CURSOR_RIGHT_SIDE',
	'GLUT_CURSOR_SPRAY',
	'GLUT_CURSOR_TEXT',
	'GLUT_CURSOR_TOP_LEFT_CORNER',
	'GLUT_CURSOR_TOP_RIGHT_CORNER',
	'GLUT_CURSOR_TOP_SIDE',
	'GLUT_CURSOR_UP_DOWN',
	'GLUT_CURSOR_WAIT',
	'GLUT_DEPTH',
	'GLUT_DEVICE_IGNORE_KEY_REPEAT',
	'GLUT_DEVICE_KEY_REPEAT',
	'GLUT_DISPLAY_MODE_POSSIBLE',
	'GLUT_DOUBLE',
	'GLUT_DOWN',
	'GLUT_ELAPSED_TIME',
	'GLUT_ENTERED',
	'GLUT_FULLY_COVERED',
	'GLUT_FULLY_RETAINED',
	'GLUT_GAME_MODE_ACTIVE',
	'GLUT_GAME_MODE_DISPLAY_CHANGED',
	'GLUT_GAME_MODE_HEIGHT',
	'GLUT_GAME_MODE_PIXEL_DEPTH',
	'GLUT_GAME_MODE_POSSIBLE',
	'GLUT_GAME_MODE_REFRESH_RATE',
	'GLUT_GAME_MODE_WIDTH',
	'GLUT_GREEN',
	'GLUT_HAS_DIAL_AND_BUTTON_BOX',
	'GLUT_HAS_JOYSTICK',
	'GLUT_HAS_KEYBOARD',
	'GLUT_HAS_MOUSE',
	'GLUT_HAS_OVERLAY',
	'GLUT_HAS_SPACEBALL',
	'GLUT_HAS_TABLET',
	'GLUT_HIDDEN',
	'GLUT_INDEX',
	'GLUT_INIT_DISPLAY_MODE',
	'GLUT_INIT_STATE',
	'GLUT_INIT_WINDOW_HEIGHT',
	'GLUT_INIT_WINDOW_WIDTH',
	'GLUT_INIT_WINDOW_X',
	'GLUT_INIT_WINDOW_Y',
	'GLUT_JOYSTICK_AXES',
	'GLUT_JOYSTICK_BUTTONS',
	'GLUT_JOYSTICK_BUTTON_A',
	'GLUT_JOYSTICK_BUTTON_B',
	'GLUT_JOYSTICK_BUTTON_C',
	'GLUT_JOYSTICK_BUTTON_D',
	'GLUT_JOYSTICK_POLL_RATE',
	'GLUT_KEY_DOWN',
	'GLUT_KEY_END',
	'GLUT_KEY_F1',
	'GLUT_KEY_F10',
	'GLUT_KEY_F11',
	'GLUT_KEY_F12',
	'GLUT_KEY_F2',
	'GLUT_KEY_F3',
	'GLUT_KEY_F4',
	'GLUT_KEY_F5',
	'GLUT_KEY_F6',
	'GLUT_KEY_F7',
	'GLUT_KEY_F8',
	'GLUT_KEY_F9',
	'GLUT_KEY_HOME',
	'GLUT_KEY_INSERT',
	'GLUT_KEY_LEFT',
	'GLUT_KEY_PAGE_DOWN',
	'GLUT_KEY_PAGE_UP',
	'GLUT_KEY_REPEAT_DEFAULT',
	'GLUT_KEY_REPEAT_OFF',
	'GLUT_KEY_REPEAT_ON',
	'GLUT_KEY_RIGHT',
	'GLUT_KEY_UP',
	'GLUT_LAYER_IN_USE',
	'GLUT_LEFT',
	'GLUT_LEFT_BUTTON',
	'GLUT_LUMINANCE',
	'GLUT_MENU_IN_USE',
	'GLUT_MENU_NOT_IN_USE',
	'GLUT_MENU_NUM_ITEMS',
	'GLUT_MIDDLE_BUTTON',
	'GLUT_MULTISAMPLE',
	'GLUT_NORMAL',
	'GLUT_NORMAL_DAMAGED',
	'GLUT_NOT_VISIBLE',
	'GLUT_NUM_BUTTON_BOX_BUTTONS',
	'GLUT_NUM_DIALS',
	'GLUT_NUM_MOUSE_BUTTONS',
	'GLUT_NUM_SPACEBALL_BUTTONS',
	'GLUT_NUM_TABLET_BUTTONS',
	'GLUT_OVERLAY',
	'GLUT_OVERLAY_DAMAGED',
	'GLUT_OVERLAY_POSSIBLE',
	'GLUT_OWNS_JOYSTICK',
	'GLUT_PARTIALLY_RETAINED',
	'GLUT_RED',
	'GLUT_RGB',
	'GLUT_RGBA',
	'GLUT_RIGHT_BUTTON',
	'GLUT_SCREEN_HEIGHT',
	'GLUT_SCREEN_HEIGHT_MM',
	'GLUT_SCREEN_WIDTH',
	'GLUT_SCREEN_WIDTH_MM',
	'GLUT_SINGLE',
	'GLUT_STENCIL',
	'GLUT_STEREO',
	'GLUT_TRANSPARENT_INDEX',
	'GLUT_UP',
	'GLUT_VIDEO_RESIZE_HEIGHT',
	'GLUT_VIDEO_RESIZE_HEIGHT_DELTA',
	'GLUT_VIDEO_RESIZE_IN_USE',
	'GLUT_VIDEO_RESIZE_POSSIBLE',
	'GLUT_VIDEO_RESIZE_WIDTH',
	'GLUT_VIDEO_RESIZE_WIDTH_DELTA',
	'GLUT_VIDEO_RESIZE_X',
	'GLUT_VIDEO_RESIZE_X_DELTA',
	'GLUT_VIDEO_RESIZE_Y',
	'GLUT_VIDEO_RESIZE_Y_DELTA',
	'GLUT_VISIBLE',
	'GLUT_WINDOW_ACCUM_ALPHA_SIZE',
	'GLUT_WINDOW_ACCUM_BLUE_SIZE',
	'GLUT_WINDOW_ACCUM_GREEN_SIZE',
	'GLUT_WINDOW_ACCUM_RED_SIZE',
	'GLUT_WINDOW_ALPHA_SIZE',
	'GLUT_WINDOW_BLUE_SIZE',
	'GLUT_WINDOW_BUFFER_SIZE',
	'GLUT_WINDOW_COLORMAP_SIZE',
	'GLUT_WINDOW_CURSOR',
	'GLUT_WINDOW_DEPTH_SIZE',
	'GLUT_WINDOW_DOUBLEBUFFER',
	'GLUT_WINDOW_FORMAT_ID',
	'GLUT_WINDOW_GREEN_SIZE',
	'GLUT_WINDOW_HEIGHT',
	'GLUT_WINDOW_NUM_CHILDREN',
	'GLUT_WINDOW_NUM_SAMPLES',
	'GLUT_WINDOW_PARENT',
	'GLUT_WINDOW_RED_SIZE',
	'GLUT_WINDOW_RGBA',
	'GLUT_WINDOW_STENCIL_SIZE',
	'GLUT_WINDOW_STEREO',
	'GLUT_WINDOW_WIDTH',
	'GLUT_WINDOW_X',
	'GLUT_WINDOW_Y',
	'GLUT_XLIB_IMPLEMENTATION',
	'GLdouble',
	'GLenum',
	'GLfloat',
	'GLint',
	'glutAddMenuEntry',
	'glutAddSubMenu',
	'glutAttachMenu',
	'glutBitmapCharacter',
	'glutBitmapLength',
	'glutBitmapWidth',
	'glutButtonBoxFunc',
	'glutChangeToMenuEntry',
	'glutChangeToSubMenu',
	'glutCopyColormap',
	'glutCreateMenu',
	'glutCreateSubWindow',
	'glutCreateWindow',
	'glutDestroyMenu',
	'glutDestroyWindow',
	'glutDetachMenu',
	'glutDeviceGet',
	'glutDialsFunc',
	'glutDisplayFunc',
	'glutEnterGameMode',
	'glutEntryFunc',
	'glutEstablishOverlay',
	'glutExtensionSupported',
	'glutForceJoystickFunc',
	'glutFullScreen',
	'glutGameModeGet',
	'glutGameModeString',
	'glutGet',
	'glutGetColor',
	'glutGetMenu',
	'glutGetModifiers',
	'glutGetWindow',
	'glutHideOverlay',
	'glutHideWindow',
	'glutIconifyWindow',
	'glutIdleFunc',
	'glutIgnoreKeyRepeat',
	'glutInit',
	'glutInitDisplayMode',
	'glutInitDisplayString',
	'glutInitWindowPosition',
	'glutInitWindowSize',
	'glutJoystickFunc',
	'glutKeyboardFunc',
	'glutKeyboardUpFunc',
	'glutLayerGet',
	'glutLeaveGameMode',
	'glutMainLoop',
	'glutMenuStateFunc',
	'glutMenuStatusFunc',
	'glutMotionFunc',
	'glutMouseFunc',
	'glutOverlayDisplayFunc',
	'glutPassiveMotionFunc',
	'glutPopWindow',
	'glutPositionWindow',
	'glutPostOverlayRedisplay',
	'glutPostRedisplay',
	'glutPostWindowOverlayRedisplay',
	'glutPostWindowRedisplay',
	'glutPushWindow',
	'glutRemoveMenuItem',
	'glutRemoveOverlay',
	'glutReportErrors',
	'glutReshapeFunc',
	'glutReshapeWindow',
	'glutSetColor',
	'glutSetCursor',
	'glutSetIconTitle',
	'glutSetKeyRepeat',
	'glutSetMenu',
	'glutSetWindow',
	'glutSetWindowTitle',
	'glutSetupVideoResizing',
	'glutShowOverlay',
	'glutShowWindow',
	'glutSolidCone',
	'glutSolidCube',
	'glutSolidDodecahedron',
	'glutSolidIcosahedron',
	'glutSolidOctahedron',
	'glutSolidSphere',
	'glutSolidTeapot',
	'glutSolidTetrahedron',
	'glutSolidTorus',
	'glutSpaceballButtonFunc',
	'glutSpaceballMotionFunc',
	'glutSpaceballRotateFunc',
	'glutSpecialFunc',
	'glutSpecialUpFunc',
	'glutStopVideoResizing',
	'glutStrokeCharacter',
	'glutStrokeLength',
	'glutStrokeWidth',
	'glutSwapBuffers',
	'glutTabletButtonFunc',
	'glutTabletMotionFunc',
	'glutTimerFunc',
	'glutUseLayer',
	'glutVideoPan',
	'glutVideoResize',
	'glutVideoResizeGet',
	'glutVisibilityFunc',
	'glutWarpPointer',
	'glutWindowStatusFunc',
	'glutWireCone',
	'glutWireCube',
	'glutWireDodecahedron',
	'glutWireIcosahedron',
	'glutWireOctahedron',
	'glutWireSphere',
	'glutWireTeapot',
	'glutWireTetrahedron',
	'glutWireTorus'
]




"""ctypes single-value "parameter" arguments (e.g. byref)
"""
REGISTRY_NAME = 'ctypeparameter'
import ctypes, _ctypes

from OpenGL import constants, constant
from OpenGL.arrays import formathandler
import operator

c = ctypes.c_float(0)
ParamaterType = ctypes.byref(c).__class__
del c

class CtypesParameterHandler( formathandler.FormatHandler ):
	"""Ctypes Paramater-type-specific data-type handler for OpenGL"""
	HANDLED_TYPES = (ParamaterType, _ctypes._SimpleCData)
	def from_param( cls, value ):
		if isinstance( value, ParamaterType ):
			return value 
		else:
			return ctypes.byref( value )
	from_param = voidDataPointer = classmethod( from_param )
	def dataPointer( cls, value ):
		if isinstance( value, ParamaterType ):
			return value 
		else:
			return ctypes.addressof( value )
	dataPointer = classmethod( dataPointer )
	def zeros( self, dims, typeCode ):
		"""Return Numpy array of zeros in given size"""
		type = GL_TYPE_TO_ARRAY_MAPPING[ typeCode ]
		for dim in dims:
			type *= dim 
		return type() # should expicitly set to 0s
	def ones( self, dims, typeCode='d' ):
		"""Return numpy array of ones in given size"""
		raise NotImplementedError( """Haven't got a good ones implementation yet""" )
##		type = GL_TYPE_TO_ARRAY_MAPPING[ typeCode ]
##		for dim in dims:
##			type *= dim 
##		return type() # should expicitly set to 0s
	def arrayToGLType( self, value ):
		"""Given a value, guess OpenGL type of the corresponding pointer"""
		if isinstance( value, ParamaterType ):
			value = value._obj
		result = ARRAY_TO_GL_TYPE_MAPPING.get( value._type_ )
		if result is not None:
			return result
		raise TypeError(
			"""Don't know GL type for array of type %r, known types: %s\nvalue:%s"""%(
				value._type_, ARRAY_TO_GL_TYPE_MAPPING.keys(), value,
			)
		)
	def arraySize( self, value, typeCode = None ):
		"""Given a data-value, calculate dimensions for the array"""
		if isinstance( value, ParamaterType ):
			value = value._obj
		dims = 1
		for base in self.types( value ):
			length = getattr( base, '_length_', None)
			if length is not None:
				dims *= length
		return dims 
	def arrayByteCount( self, value, typeCode = None ):
		"""Given a data-value, calculate number of bytes required to represent"""
		if isinstance( value, ParamaterType ):
			value = value._obj
		return ctypes.sizeof( value )
	def types( self, value ):
		"""Produce iterable producing all composite types"""
		if isinstance( value, ParamaterType ):
			value = value._obj
		dimObject = value
		while dimObject is not None:
			yield dimObject
			dimObject = getattr( dimObject, '_type_', None )
			if isinstance( dimObject, (str,unicode)):
				dimObject = None 
	def dims( self, value ):
		"""Produce iterable of all dimensions"""
		if isinstance( value, ParamaterType ):
			value = value._obj
		for base in self.types( value ):
			length = getattr( base, '_length_', None)
			if length is not None:
				yield length
	def asArray( self, value, typeCode=None ):
		"""Convert given value to an array value of given typeCode"""
		if isinstance( value, ParamaterType ):
			value = value._obj
		return ctypes.byref( value )
	def unitSize( self, value, typeCode=None ):
		"""Determine unit size of an array (if possible)"""
		if isinstance( value, ParamaterType ):
			value = value._obj
		return tuple(self.dims(value))[-1]
	def dimensions( self, value, typeCode=None ):
		"""Determine dimensions of the passed array value (if possible)"""
		if isinstance( value, ParamaterType ):
			value = value._obj
		return tuple( self.dims(value) )


ARRAY_TO_GL_TYPE_MAPPING = {
	constants.GLdouble: constants.GL_DOUBLE,
	constants.GLfloat: constants.GL_FLOAT,
	constants.GLint: constants.GL_INT,
	constants.GLuint: constants.GL_UNSIGNED_INT,
	constants.GLshort: constants.GL_SHORT,
	constants.GLushort: constants.GL_UNSIGNED_SHORT,
		
	constants.GLchar: constants.GL_CHAR,
	constants.GLbyte: constants.GL_BYTE,
	constants.GLubyte: constants.GL_UNSIGNED_BYTE,
}
GL_TYPE_TO_ARRAY_MAPPING = {
	constants.GL_DOUBLE: constants.GLdouble,
	constants.GL_FLOAT: constants.GLfloat,
	constants.GL_INT: constants.GLint,
	constants.GL_UNSIGNED_INT: constants.GLuint,
	constants.GL_SHORT: constants.GLshort,
	constants.GL_UNSIGNED_SHORT: constants.GLushort,
		
	constants.GL_CHAR: constants.GLchar,
	constants.GL_BYTE: constants.GLbyte,
	constants.GL_UNSIGNED_BYTE: constants.GLubyte,
}



'''OpenGL extension ARB.draw_buffers

Overview (from the spec)
	
	This extension extends ARB_fragment_program and ARB_fragment_shader
	to allow multiple output colors, and provides a mechanism for
	directing those outputs to multiple color buffers.
	

The official definition of this extension is available here:
	http://oss.sgi.com/projects/ogl-sample/registry/ARB/draw_buffers.txt

Automatically generated by the get_gl_extensions script, do not edit!
'''
from OpenGL import platform, constants, constant, arrays
from OpenGL import extensions
from OpenGL.GL import glget
import ctypes
EXTENSION_NAME = 'GL_ARB_draw_buffers'
GL_MAX_DRAW_BUFFERS_ARB = constant.Constant( 'GL_MAX_DRAW_BUFFERS_ARB', 0x8824 )
glget.addGLGetConstant( GL_MAX_DRAW_BUFFERS_ARB, (1,) )
GL_DRAW_BUFFER0_ARB = constant.Constant( 'GL_DRAW_BUFFER0_ARB', 0x8825 )
GL_DRAW_BUFFER1_ARB = constant.Constant( 'GL_DRAW_BUFFER1_ARB', 0x8826 )
GL_DRAW_BUFFER2_ARB = constant.Constant( 'GL_DRAW_BUFFER2_ARB', 0x8827 )
GL_DRAW_BUFFER3_ARB = constant.Constant( 'GL_DRAW_BUFFER3_ARB', 0x8828 )
GL_DRAW_BUFFER4_ARB = constant.Constant( 'GL_DRAW_BUFFER4_ARB', 0x8829 )
GL_DRAW_BUFFER5_ARB = constant.Constant( 'GL_DRAW_BUFFER5_ARB', 0x882A )
GL_DRAW_BUFFER6_ARB = constant.Constant( 'GL_DRAW_BUFFER6_ARB', 0x882B )
GL_DRAW_BUFFER7_ARB = constant.Constant( 'GL_DRAW_BUFFER7_ARB', 0x882C )
GL_DRAW_BUFFER8_ARB = constant.Constant( 'GL_DRAW_BUFFER8_ARB', 0x882D )
GL_DRAW_BUFFER9_ARB = constant.Constant( 'GL_DRAW_BUFFER9_ARB', 0x882E )
GL_DRAW_BUFFER10_ARB = constant.Constant( 'GL_DRAW_BUFFER10_ARB', 0x882F )
GL_DRAW_BUFFER11_ARB = constant.Constant( 'GL_DRAW_BUFFER11_ARB', 0x8830 )
GL_DRAW_BUFFER12_ARB = constant.Constant( 'GL_DRAW_BUFFER12_ARB', 0x8831 )
GL_DRAW_BUFFER13_ARB = constant.Constant( 'GL_DRAW_BUFFER13_ARB', 0x8832 )
GL_DRAW_BUFFER14_ARB = constant.Constant( 'GL_DRAW_BUFFER14_ARB', 0x8833 )
GL_DRAW_BUFFER15_ARB = constant.Constant( 'GL_DRAW_BUFFER15_ARB', 0x8834 )
glDrawBuffersARB = platform.createExtensionFunction( 
	'glDrawBuffersARB', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLsizei, arrays.GLuintArray,),
	doc = 'glDrawBuffersARB( GLsizei(n), GLuintArray(bufs) ) -> None',
	argNames = ('n', 'bufs',),
)


def glInitDrawBuffersARB():
	'''Return boolean indicating whether this extension is available'''
	return extensions.hasGLExtension( EXTENSION_NAME )



'''OpenGL extension ARB.texture_border_clamp

Overview (from the spec)
	
	The base OpenGL provides clamping such that the texture coordinates are
	limited to exactly the range [0,1].  When a texture coordinate is clamped
	using this algorithm, the texture sampling filter straddles the edge of
	the texture image, taking 1/2 its sample values from within the texture
	image, and the other 1/2 from the texture border.  It is sometimes
	desirable for a texture to be clamped to the border color, rather than to
	an average of the border and edge colors.
	
	This extension defines an additional texture clamping algorithm.
	CLAMP_TO_BORDER_ARB clamps texture coordinates at all mipmap levels such
	that NEAREST and LINEAR filters return only the color of the border
	texels.

The official definition of this extension is available here:
	http://oss.sgi.com/projects/ogl-sample/registry/ARB/texture_border_clamp.txt

Automatically generated by the get_gl_extensions script, do not edit!
'''
from OpenGL import platform, constants, constant, arrays
from OpenGL import extensions
from OpenGL.GL import glget
import ctypes
EXTENSION_NAME = 'GL_ARB_texture_border_clamp'
GL_CLAMP_TO_BORDER_ARB = constant.Constant( 'GL_CLAMP_TO_BORDER_ARB', 0x812D )


def glInitTextureBorderClampARB():
	'''Return boolean indicating whether this extension is available'''
	return extensions.hasGLExtension( EXTENSION_NAME )



'''OpenGL extension NV.framebuffer_multisample_coverage

The official definition of this extension is available here:
	http://oss.sgi.com/projects/ogl-sample/registry/NV/framebuffer_multisample_coverage.txt

Automatically generated by the get_gl_extensions script, do not edit!
'''
from OpenGL import platform, constants, constant, arrays
from OpenGL import extensions
from OpenGL.GL import glget
import ctypes
EXTENSION_NAME = 'GL_NV_framebuffer_multisample_coverage'
GL_RENDERBUFFER_COVERAGE_SAMPLES_NV = constant.Constant( 'GL_RENDERBUFFER_COVERAGE_SAMPLES_NV', 0x8CAB )
GL_RENDERBUFFER_COLOR_SAMPLES_NV = constant.Constant( 'GL_RENDERBUFFER_COLOR_SAMPLES_NV', 0x8E10 )
GL_MAX_MULTISAMPLE_COVERAGE_MODES_NV = constant.Constant( 'GL_MAX_MULTISAMPLE_COVERAGE_MODES_NV', 0x8E11 )
GL_MULTISAMPLE_COVERAGE_MODES_NV = constant.Constant( 'GL_MULTISAMPLE_COVERAGE_MODES_NV', 0x8E12 )
glRenderbufferStorageMultisampleCoverageNV = platform.createExtensionFunction( 
	'glRenderbufferStorageMultisampleCoverageNV', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLenum, constants.GLsizei, constants.GLsizei, constants.GLenum, constants.GLsizei, constants.GLsizei,),
	doc = 'glRenderbufferStorageMultisampleCoverageNV( GLenum(target), GLsizei(coverageSamples), GLsizei(colorSamples), GLenum(internalformat), GLsizei(width), GLsizei(height) ) -> None',
	argNames = ('target', 'coverageSamples', 'colorSamples', 'internalformat', 'width', 'height',),
)


def glInitFramebufferMultisampleCoverageNV():
	'''Return boolean indicating whether this extension is available'''
	return extensions.hasGLExtension( EXTENSION_NAME )



'''OpenGL extension EXT.timer_query

The official definition of this extension is available here:
	http://oss.sgi.com/projects/ogl-sample/registry/EXT/timer_query.txt

Automatically generated by the get_gl_extensions script, do not edit!
'''
from OpenGL import platform, constants, constant, arrays
from OpenGL import extensions
from OpenGL.GL import glget
import ctypes
EXTENSION_NAME = 'GL_EXT_timer_query'
GL_TIME_ELAPSED_EXT = constant.Constant( 'GL_TIME_ELAPSED_EXT', 0x88BF )
glGetQueryObjecti64vEXT = platform.createExtensionFunction( 
	'glGetQueryObjecti64vEXT', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLuint, constants.GLenum, ctypes.POINTER(constants.GLint64EXT),),
	doc = 'glGetQueryObjecti64vEXT( GLuint(id), GLenum(pname), POINTER(constants.GLint64EXT)(params) ) -> None',
	argNames = ('id', 'pname', 'params',),
)

glGetQueryObjectui64vEXT = platform.createExtensionFunction( 
	'glGetQueryObjectui64vEXT', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLuint, constants.GLenum, ctypes.POINTER(constants.GLuint64EXT),),
	doc = 'glGetQueryObjectui64vEXT( GLuint(id), GLenum(pname), POINTER(constants.GLuint64EXT)(params) ) -> None',
	argNames = ('id', 'pname', 'params',),
)


def glInitTimerQueryEXT():
	'''Return boolean indicating whether this extension is available'''
	return extensions.hasGLExtension( EXTENSION_NAME )



'''OpenGL extension SGIX.shadow_ambient

Overview (from the spec)
	
	This extension allows a value other than 0.0 to be returned by the
	SGIX_shadow operation in the case when the shadow test passes.
	With this extension any floating point value in the range [0.0,
	1.0] can be returned as the texture value when an object is in
	shadow.  This allows the (untextured) ambient lighting and direct
	shadowed lighting from a single light source to be computed in a
	single pass.

The official definition of this extension is available here:
	http://oss.sgi.com/projects/ogl-sample/registry/SGIX/shadow_ambient.txt

Automatically generated by the get_gl_extensions script, do not edit!
'''
from OpenGL import platform, constants, constant, arrays
from OpenGL import extensions
from OpenGL.GL import glget
import ctypes
EXTENSION_NAME = 'GL_SGIX_shadow_ambient'
GL_SHADOW_AMBIENT_SGIX = constant.Constant( 'GL_SHADOW_AMBIENT_SGIX', 0x80BF )


def glInitShadowAmbientSGIX():
	'''Return boolean indicating whether this extension is available'''
	return extensions.hasGLExtension( EXTENSION_NAME )



'''OpenGL extension NV.parameter_buffer_object

The official definition of this extension is available here:
	http://oss.sgi.com/projects/ogl-sample/registry/NV/parameter_buffer_object.txt

Automatically generated by the get_gl_extensions script, do not edit!
'''
from OpenGL import platform, constants, constant, arrays
from OpenGL import extensions
from OpenGL.GL import glget
import ctypes
EXTENSION_NAME = 'GL_NV_parameter_buffer_object'
GL_MAX_PROGRAM_PARAMETER_BUFFER_BINDINGS_NV = constant.Constant( 'GL_MAX_PROGRAM_PARAMETER_BUFFER_BINDINGS_NV', 0x8DA0 )
GL_MAX_PROGRAM_PARAMETER_BUFFER_SIZE_NV = constant.Constant( 'GL_MAX_PROGRAM_PARAMETER_BUFFER_SIZE_NV', 0x8DA1 )
GL_VERTEX_PROGRAM_PARAMETER_BUFFER_NV = constant.Constant( 'GL_VERTEX_PROGRAM_PARAMETER_BUFFER_NV', 0x8DA2 )
GL_GEOMETRY_PROGRAM_PARAMETER_BUFFER_NV = constant.Constant( 'GL_GEOMETRY_PROGRAM_PARAMETER_BUFFER_NV', 0x8DA3 )
GL_FRAGMENT_PROGRAM_PARAMETER_BUFFER_NV = constant.Constant( 'GL_FRAGMENT_PROGRAM_PARAMETER_BUFFER_NV', 0x8DA4 )
glProgramBufferParametersfvNV = platform.createExtensionFunction( 
	'glProgramBufferParametersfvNV', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLenum, constants.GLuint, constants.GLuint, constants.GLsizei, arrays.GLfloatArray,),
	doc = 'glProgramBufferParametersfvNV( GLenum(target), GLuint(buffer), GLuint(index), GLsizei(count), GLfloatArray(params) ) -> None',
	argNames = ('target', 'buffer', 'index', 'count', 'params',),
)

glProgramBufferParametersIivNV = platform.createExtensionFunction( 
	'glProgramBufferParametersIivNV', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLenum, constants.GLuint, constants.GLuint, constants.GLsizei, arrays.GLintArray,),
	doc = 'glProgramBufferParametersIivNV( GLenum(target), GLuint(buffer), GLuint(index), GLsizei(count), GLintArray(params) ) -> None',
	argNames = ('target', 'buffer', 'index', 'count', 'params',),
)

glProgramBufferParametersIuivNV = platform.createExtensionFunction( 
	'glProgramBufferParametersIuivNV', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLenum, constants.GLuint, constants.GLuint, constants.GLsizei, arrays.GLuintArray,),
	doc = 'glProgramBufferParametersIuivNV( GLenum(target), GLuint(buffer), GLuint(index), GLsizei(count), GLuintArray(params) ) -> None',
	argNames = ('target', 'buffer', 'index', 'count', 'params',),
)


def glInitParameterBufferObjectNV():
	'''Return boolean indicating whether this extension is available'''
	return extensions.hasGLExtension( EXTENSION_NAME )



'''OpenGL extension APPLE.specular_vector

This module customises the behaviour of the 
OpenGL.raw.GL.APPLE.specular_vector to provide a more 
Python-friendly API
'''
from OpenGL import platform, constants, constant, arrays
from OpenGL import extensions, wrapper
from OpenGL.GL import glget
import ctypes
from OpenGL.raw.GL.APPLE.specular_vector import *
### END AUTOGENERATED SECTION
from OpenGL.GL import glget
glget.addGLGetConstant( GL_LIGHT_MODEL_SPECULAR_VECTOR_APPLE, (1,) ) # check size...



"""String-array-handling code for PyOpenGL
"""
from OpenGL.arrays._strings import dataPointer
from OpenGL import constants
from OpenGL.arrays import formathandler
import ctypes

class StringHandler( formathandler.FormatHandler ):
	"""String-specific data-type handler for OpenGL"""
	HANDLED_TYPES = (str, )
	from_param = staticmethod( dataPointer )
	dataPointer = staticmethod( dataPointer )
	def zeros( self, dims, typeCode=None ):
		"""Currently don't allow strings as output types!"""
		raise NotImplemented( """Don't currently support strings as output arrays""" )
	def ones( self, dims, typeCode=None ):
		"""Currently don't allow strings as output types!"""
		raise NotImplemented( """Don't currently support strings as output arrays""" )
	def arrayToGLType( self, value ):
		"""Given a value, guess OpenGL type of the corresponding pointer"""
		raise NotImplemented( """Can't guess data-type from a string-type argument""" )
	def arraySize( self, value, typeCode = None ):
		"""Given a data-value, calculate ravelled size for the array"""
		# need to get bits-per-element...
		byteCount = BYTE_SIZES[ typeCode ]
		return len(value)//byteCount
	def arrayByteCount( self, value, typeCode = None ):
		"""Given a data-value, calculate number of bytes required to represent"""
		return len(value)
	def asArray( self, value, typeCode=None ):
		"""Convert given value to an array value of given typeCode"""
		if isinstance( value, str ):
			return value
		elif hasattr( value, 'tostring' ):
			return value.tostring()
		elif hasattr( value, 'raw' ):
			return value.raw
		# could convert types to string here, but we're not registered for 
		# anything save string types...
		raise TypeError( """String handler got non-string object: %r"""%(type(value)))
	def dimensions( self, value, typeCode=None ):
		"""Determine dimensions of the passed array value (if possible)"""
		raise TypeError(
			"""Cannot calculate dimensions for a String data-type"""
		)

BYTE_SIZES = {
	constants.GL_DOUBLE: ctypes.sizeof( constants.GLdouble ),
	constants.GL_FLOAT: ctypes.sizeof( constants.GLfloat ),
	constants.GL_INT: ctypes.sizeof( constants.GLint ),
	constants.GL_SHORT: ctypes.sizeof( constants.GLshort ),
	constants.GL_UNSIGNED_BYTE: ctypes.sizeof( constants.GLubyte ),
	constants.GL_UNSIGNED_SHORT: ctypes.sizeof( constants.GLshort ),
	constants.GL_BYTE: ctypes.sizeof( constants.GLbyte ),
	constants.GL_UNSIGNED_INT: ctypes.sizeof( constants.GLuint ),
}



'''OpenGL extension SGIS.point_parameters

The official definition of this extension is available here:
	http://oss.sgi.com/projects/ogl-sample/registry/SGIS/point_parameters.txt

Automatically generated by the get_gl_extensions script, do not edit!
'''
from OpenGL import platform, constants, constant, arrays
from OpenGL import extensions
from OpenGL.GL import glget
import ctypes
EXTENSION_NAME = 'GL_SGIS_point_parameters'
GL_POINT_SIZE_MIN_SGIS = constant.Constant( 'GL_POINT_SIZE_MIN_SGIS', 0x8126 )
GL_POINT_SIZE_MAX_SGIS = constant.Constant( 'GL_POINT_SIZE_MAX_SGIS', 0x8127 )
GL_POINT_FADE_THRESHOLD_SIZE_SGIS = constant.Constant( 'GL_POINT_FADE_THRESHOLD_SIZE_SGIS', 0x8128 )
GL_DISTANCE_ATTENUATION_SGIS = constant.Constant( 'GL_DISTANCE_ATTENUATION_SGIS', 0x8129 )
glPointParameterfSGIS = platform.createExtensionFunction( 
	'glPointParameterfSGIS', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLenum, constants.GLfloat,),
	doc = 'glPointParameterfSGIS( GLenum(pname), GLfloat(param) ) -> None',
	argNames = ('pname', 'param',),
)

glPointParameterfvSGIS = platform.createExtensionFunction( 
	'glPointParameterfvSGIS', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLenum, arrays.GLfloatArray,),
	doc = 'glPointParameterfvSGIS( GLenum(pname), GLfloatArray(params) ) -> None',
	argNames = ('pname', 'params',),
)


def glInitPointParametersSGIS():
	'''Return boolean indicating whether this extension is available'''
	return extensions.hasGLExtension( EXTENSION_NAME )



'''OpenGL extension EXT.texture_object

Overview (from the spec)
	
	This extension introduces named texture objects.  The only way to name
	a texture in GL 1.0 is by defining it as a single display list.  Because
	display lists cannot be edited, these objects are static.  Yet it is
	important to be able to change the images and parameters of a texture.

The official definition of this extension is available here:
	http://oss.sgi.com/projects/ogl-sample/registry/EXT/texture_object.txt

Automatically generated by the get_gl_extensions script, do not edit!
'''
from OpenGL import platform, constants, constant, arrays
from OpenGL import extensions
from OpenGL.GL import glget
import ctypes
EXTENSION_NAME = 'GL_EXT_texture_object'
GL_TEXTURE_PRIORITY_EXT = constant.Constant( 'GL_TEXTURE_PRIORITY_EXT', 0x8066 )
GL_TEXTURE_RESIDENT_EXT = constant.Constant( 'GL_TEXTURE_RESIDENT_EXT', 0x8067 )
GL_TEXTURE_1D_BINDING_EXT = constant.Constant( 'GL_TEXTURE_1D_BINDING_EXT', 0x8068 )
glget.addGLGetConstant( GL_TEXTURE_1D_BINDING_EXT, (1,) )
GL_TEXTURE_2D_BINDING_EXT = constant.Constant( 'GL_TEXTURE_2D_BINDING_EXT', 0x8069 )
glget.addGLGetConstant( GL_TEXTURE_2D_BINDING_EXT, (1,) )
GL_TEXTURE_3D_BINDING_EXT = constant.Constant( 'GL_TEXTURE_3D_BINDING_EXT', 0x806A )
glget.addGLGetConstant( GL_TEXTURE_3D_BINDING_EXT, (1,) )
glAreTexturesResidentEXT = platform.createExtensionFunction( 
	'glAreTexturesResidentEXT', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=constants.GLboolean, 
	argTypes=(constants.GLsizei, arrays.GLuintArray, ctypes.POINTER(constants.GLboolean),),
	doc = 'glAreTexturesResidentEXT( GLsizei(n), GLuintArray(textures), POINTER(constants.GLboolean)(residences) ) -> constants.GLboolean',
	argNames = ('n', 'textures', 'residences',),
)

glBindTextureEXT = platform.createExtensionFunction( 
	'glBindTextureEXT', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLenum, constants.GLuint,),
	doc = 'glBindTextureEXT( GLenum(target), GLuint(texture) ) -> None',
	argNames = ('target', 'texture',),
)

glDeleteTexturesEXT = platform.createExtensionFunction( 
	'glDeleteTexturesEXT', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLsizei, arrays.GLuintArray,),
	doc = 'glDeleteTexturesEXT( GLsizei(n), GLuintArray(textures) ) -> None',
	argNames = ('n', 'textures',),
)

glGenTexturesEXT = platform.createExtensionFunction( 
	'glGenTexturesEXT', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLsizei, arrays.GLuintArray,),
	doc = 'glGenTexturesEXT( GLsizei(n), GLuintArray(textures) ) -> None',
	argNames = ('n', 'textures',),
)

glIsTextureEXT = platform.createExtensionFunction( 
	'glIsTextureEXT', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=constants.GLboolean, 
	argTypes=(constants.GLuint,),
	doc = 'glIsTextureEXT( GLuint(texture) ) -> constants.GLboolean',
	argNames = ('texture',),
)

glPrioritizeTexturesEXT = platform.createExtensionFunction( 
	'glPrioritizeTexturesEXT', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLsizei, arrays.GLuintArray, arrays.GLclampfArray,),
	doc = 'glPrioritizeTexturesEXT( GLsizei(n), GLuintArray(textures), GLclampfArray(priorities) ) -> None',
	argNames = ('n', 'textures', 'priorities',),
)


def glInitTextureObjectEXT():
	'''Return boolean indicating whether this extension is available'''
	return extensions.hasGLExtension( EXTENSION_NAME )



"""Implementation of OpenGL constant objects"""
import sys

class Constant( object ):
	"""OpenGL constant that displays itself as a name rather than a value
	
	The purpose of this class is to make debugging OpenGL code easier,
	as you recieve messages that say what value you passed in in a 
	human-readable form, rather than as a bald number that requires
	lookup and disambiguation in the header file.
	"""
	def __new__( cls, name, value ):
		"""Initialise the constant with the given name and value"""
		if isinstance( value, float ) and cls is not FloatConstant:
			return FloatConstant( name, value )
		elif isinstance( value, (int,long) ) and cls is not IntConstant:
			return IntConstant( name, value )
		elif isinstance( value, (str,unicode) ) and cls is not StringConstant:
			return StringConstant( name, str(value) )
		if isinstance( value, long ):
			if value > sys.maxint:
				value = - (value & sys.maxint)
		base = super(Constant,cls).__new__( cls, value )
		base.name = name 
		return base
	def __repr__( self ):
		"""Return the name, rather than the bald value"""
		return self.name 

class NumericConstant( Constant ):
	"""Base class for numeric-value constants"""
	def __str__( self ):
		"""Return the value as a human-friendly string"""
		return '%s (%s)'%(self.name,super(Constant,self).__str__())
	def __getstate__(self):
		"""Retrieve state for pickle and the like"""
		return self.name
	def __setstate__( self, state ):
		self.name = state
		
class IntConstant( NumericConstant, int ):
	"""Integer constant"""
	__slots__ = ('name',)
class FloatConstant( NumericConstant, float ):
	"""Float constant"""
	__slots__ = ('name',)

class StringConstant( Constant, str ):
	"""String constants"""
	def __repr__( self ):
		"""Return the value as a human-friendly string"""
		return '%s (%s)'%(self.name,super(Constant,self).__str__())
	def __getnewargs__( self ):
		"""Produce the new arguments for recreating the instance"""
		return (self.name,) + super( Constant, self ).__getnewargs__()

if __name__ == "__main__":
	x = IntConstant( 'testint', 3 )
	y = FloatConstant( 'testfloat', 3.0 )
	z = StringConstant( 'teststr', 'some testing string' )
	
	import pickle
	for val in x,y,z:
		restored = pickle.loads( pickle.dumps( val ))
		assert restored == val, (str(restored),str(val))
		assert restored.name == val.name, (restored.name,val.name)
		



'''OpenGL extension ATI.texture_mirror_once

Overview (from the spec)
	
	ATI_texture_mirror_once extends the set of texture wrap modes to 
	include two modes (GL_MIRROR_CLAMP_ATI, GL_MIRROR_CLAMP_TO_EDGE_ATI) 
	that effectively use a texture map twice as large as the original image 
	in which the additional half of the new image is a mirror image of the 
	original image.
	
	This new mode relaxes the need to generate images whose opposite edges
	match by using the original image to generate a matching "mirror image".
	This mode allows the texture to be mirrored only once in the negative
	s, t, and r directions.

The official definition of this extension is available here:
	http://oss.sgi.com/projects/ogl-sample/registry/ATI/texture_mirror_once.txt

Automatically generated by the get_gl_extensions script, do not edit!
'''
from OpenGL import platform, constants, constant, arrays
from OpenGL import extensions
from OpenGL.GL import glget
import ctypes
EXTENSION_NAME = 'GL_ATI_texture_mirror_once'
GL_MIRROR_CLAMP_ATI = constant.Constant( 'GL_MIRROR_CLAMP_ATI', 0x8742 )
GL_MIRROR_CLAMP_TO_EDGE_ATI = constant.Constant( 'GL_MIRROR_CLAMP_TO_EDGE_ATI', 0x8743 )


def glInitTextureMirrorOnceATI():
	'''Return boolean indicating whether this extension is available'''
	return extensions.hasGLExtension( EXTENSION_NAME )



'''OpenGL extension INGR.color_clamp

Overview (from the spec)
	
	Various RGBA color space conversions require clamping to values
	in a more constrained range than [0, 1].  This extension allows
	the definition of independent color clamp values for each of the
	four color components as part of the Final Conversion in the pixel
	transfer path for draws, reads, and copies.

The official definition of this extension is available here:
	http://oss.sgi.com/projects/ogl-sample/registry/INGR/color_clamp.txt

Automatically generated by the get_gl_extensions script, do not edit!
'''
from OpenGL import platform, constants, constant, arrays
from OpenGL import extensions
from OpenGL.GL import glget
import ctypes
EXTENSION_NAME = 'GL_INGR_color_clamp'
GL_RED_MIN_CLAMP_INGR = constant.Constant( 'GL_RED_MIN_CLAMP_INGR', 0x8560 )
glget.addGLGetConstant( GL_RED_MIN_CLAMP_INGR, (1,) )
GL_GREEN_MIN_CLAMP_INGR = constant.Constant( 'GL_GREEN_MIN_CLAMP_INGR', 0x8561 )
glget.addGLGetConstant( GL_GREEN_MIN_CLAMP_INGR, (1,) )
GL_BLUE_MIN_CLAMP_INGR = constant.Constant( 'GL_BLUE_MIN_CLAMP_INGR', 0x8562 )
glget.addGLGetConstant( GL_BLUE_MIN_CLAMP_INGR, (1,) )
GL_ALPHA_MIN_CLAMP_INGR = constant.Constant( 'GL_ALPHA_MIN_CLAMP_INGR', 0x8563 )
glget.addGLGetConstant( GL_ALPHA_MIN_CLAMP_INGR, (1,) )
GL_RED_MAX_CLAMP_INGR = constant.Constant( 'GL_RED_MAX_CLAMP_INGR', 0x8564 )
glget.addGLGetConstant( GL_RED_MAX_CLAMP_INGR, (1,) )
GL_GREEN_MAX_CLAMP_INGR = constant.Constant( 'GL_GREEN_MAX_CLAMP_INGR', 0x8565 )
glget.addGLGetConstant( GL_GREEN_MAX_CLAMP_INGR, (1,) )
GL_BLUE_MAX_CLAMP_INGR = constant.Constant( 'GL_BLUE_MAX_CLAMP_INGR', 0x8566 )
glget.addGLGetConstant( GL_BLUE_MAX_CLAMP_INGR, (1,) )
GL_ALPHA_MAX_CLAMP_INGR = constant.Constant( 'GL_ALPHA_MAX_CLAMP_INGR', 0x8567 )
glget.addGLGetConstant( GL_ALPHA_MAX_CLAMP_INGR, (1,) )


def glInitColorClampINGR():
	'''Return boolean indicating whether this extension is available'''
	return extensions.hasGLExtension( EXTENSION_NAME )



'''OpenGL extension EXT.rescale_normal

This module customises the behaviour of the 
OpenGL.raw.GL.EXT.rescale_normal to provide a more 
Python-friendly API
'''
from OpenGL import platform, constants, constant, arrays
from OpenGL import extensions, wrapper
from OpenGL.GL import glget
import ctypes
from OpenGL.raw.GL.EXT.rescale_normal import *
### END AUTOGENERATED SECTION


"""ctypes data-pointers as a data-format mechanism
"""
REGISTRY_NAME = 'ctypespointers'
import ctypes, _ctypes

from OpenGL import constants, constant
from OpenGL.arrays import formathandler
import operator

class CtypesPointerHandler( formathandler.FormatHandler ):
	"""Ctypes Pointer-type-specific data-type handler for OpenGL
	
	Because pointers do not have size information we can't use
	them for output of data, but they can be used for certain
	types of input...
	"""
	from_param = staticmethod( ctypes.byref )
	def from_param( cls, value ):
		return value
	from_param = classmethod( from_param )
##		if isinstance( value, ctypes._Pointer ):
##			return value 
##		else:
##			return ctypes.byref( value )
	dataPointer = staticmethod( ctypes.addressof )
	HANDLED_TYPES = (ctypes._Pointer, )
	def voidDataPointer( cls, value ):
		"""Given value in a known data-pointer type, return void_p for pointer"""
		return ctypes.cast( value, ctypes.c_void_p )
	def zeros( self, dims, typeCode ):
		"""Return Numpy array of zeros in given size"""
		raise NotImplementedError( """Sized output doesn't yet work...""" )
	def ones( self, dims, typeCode='d' ):
		"""Return numpy array of ones in given size"""
		raise NotImplementedError( """Haven't got a good ones implementation yet""" )
	def arrayToGLType( self, value ):
		"""Given a value, guess OpenGL type of the corresponding pointer"""
		result = ARRAY_TO_GL_TYPE_MAPPING.get( value._type_ )
		if result is not None:
			return result
		raise TypeError(
			"""Don't know GL type for array of type %r, known types: %s\nvalue:%s"""%(
				value._type_, ARRAY_TO_GL_TYPE_MAPPING.keys(), value,
			)
		)
	def arraySize( self, value, typeCode = None ):
		"""Given a data-value, calculate dimensions for the array"""
		raise NotImplementedError( """Haven't got an arraySize implementation""" )
	def asArray( self, value, typeCode=None ):
		"""Convert given value to an array value of given typeCode"""
		return value
	def unitSize( self, value, typeCode=None ):
		"""Determine unit size of an array (if possible)"""
		return 1
	def dimensions( self, value, typeCode=None ):
		"""Determine dimensions of the passed array value (if possible)"""
		raise NotImplementedError( """Haven't got a dimensions implementation""" )


ARRAY_TO_GL_TYPE_MAPPING = {
	constants.GLdouble: constants.GL_DOUBLE,
	constants.GLfloat: constants.GL_FLOAT,
	constants.GLint: constants.GL_INT,
	constants.GLuint: constants.GL_UNSIGNED_INT,
	constants.GLshort: constants.GL_SHORT,
	constants.GLushort: constants.GL_UNSIGNED_SHORT,
		
	constants.GLchar: constants.GL_CHAR,
	constants.GLbyte: constants.GL_BYTE,
	constants.GLubyte: constants.GL_UNSIGNED_BYTE,
}
GL_TYPE_TO_ARRAY_MAPPING = {
	constants.GL_DOUBLE: constants.GLdouble,
	constants.GL_FLOAT: constants.GLfloat,
	constants.GL_INT: constants.GLint,
	constants.GL_UNSIGNED_INT: constants.GLuint,
	constants.GL_SHORT: constants.GLshort,
	constants.GL_UNSIGNED_SHORT: constants.GLushort,
		
	constants.GL_CHAR: constants.GLchar,
	constants.GL_BYTE: constants.GLbyte,
	constants.GL_UNSIGNED_BYTE: constants.GLubyte,
}



"""Array data-type implementations (abstraction points for GL array types"""
import ctypes
import OpenGL
from OpenGL import constants, plugins
from OpenGL.arrays import formathandler
from OpenGL import logs
log = logs.getLog( 'OpenGL.arrays.arraydatatype' )

class ArrayDatatype( object ):
	"""Mix-in for array datatype classes
	
	The ArrayDatatype marker essentially is used to mark a particular argument
	as having an "array" type, which means that it is eligible for handling 
	via the arrays sub-package and its registered handlers.
	"""
	typeConstant = None
	def getHandler( cls, value ):
		"""Retrieve a handler for the given value, raise error on lookup failure"""
		typ = type( value )
		try:
			handler = cls.TYPE_REGISTRY.get( typ )
		except AttributeError, err:
			formathandler.FormatHandler.loadAll()
			cls.TYPE_REGISTRY = formathandler.FormatHandler.TYPE_REGISTRY
			handler = cls.TYPE_REGISTRY.get( typ )
			if handler is None: 
				handler = plugins.FormatHandler.match( typ )
		if handler is None:
			if hasattr( typ, '__mro__' ):
				for base in typ.__mro__:
					handler = cls.TYPE_REGISTRY.get( base )
					if handler is None:
						handler = plugins.FormatHandler.match( base )
					if handler:
						handler = cls.TYPE_REGISTRY[ base ]
						handler.registerEquivalent( typ, base )
						cls.TYPE_REGISTRY[ typ ] = handler 
						return handler
			raise TypeError(
				"""No array-type handler for type %r (value: %s) registered"""%(
					typ, repr(value)[:50]
				)
			)
		return handler
	getHandler = classmethod( getHandler )
	def from_param( cls, value ):
		"""Given a value in a known data-pointer type, convert to a ctypes pointer"""
		return cls.getHandler(value).from_param( value )
	from_param = classmethod( logs.logOnFail( from_param, log ) )
	def dataPointer( cls, value ):
		"""Given a value in a known data-pointer type, return long for pointer"""
		try:
			return cls.getHandler(value).dataPointer( value )
		except Exception, err:
			log.warn(
				"""Failure in from_param for %s instance %s""", type(value), value,
			)
			raise
	dataPointer = classmethod( logs.logOnFail( dataPointer, log ) )
	def voidDataPointer( cls, value ):
		"""Given value in a known data-pointer type, return void_p for pointer"""
		return ctypes.c_void_p( cls.dataPointer( value ))
	voidDataPointer = classmethod( logs.logOnFail( voidDataPointer, log ) )
	def typedPointer( cls, value ):
		"""Return a pointer-to-base-type pointer for given value"""
		return ctypes.cast( cls.dataPointer(value), ctypes.POINTER( cls.baseType ))
	typedPointer = classmethod( typedPointer )
	def asArray( cls, value, typeCode=None ):
		"""Given a value, convert to preferred array representation"""
		return cls.getHandler(value).asArray( value, typeCode or cls.typeConstant )
	asArray = classmethod( logs.logOnFail( asArray, log ) )
	def arrayToGLType( cls, value ):
		"""Given a data-value, guess the OpenGL type of the corresponding pointer"""
		return cls.getHandler(value).arrayToGLType( value )
	arrayToGLType = classmethod( logs.logOnFail( arrayToGLType, log ) )
	def arraySize( cls, value, typeCode = None ):
		"""Given a data-value, calculate dimensions for the array (number-of-units)"""
		return cls.getHandler(value).arraySize( value, typeCode or cls.typeConstant )
	arraySize = classmethod( logs.logOnFail( arraySize, log ) )
	def unitSize( cls, value, typeCode=None ):
		"""Determine unit size of an array (if possible)
		
		Uses our local type if defined, otherwise asks the handler to guess...
		"""
		return cls.getHandler(value).unitSize( value, typeCode or cls.typeConstant )
	unitSize = classmethod( logs.logOnFail( unitSize, log ) )
	def returnHandler( cls ):
		"""Get the default return-handler"""
		try:
			return formathandler.FormatHandler.RETURN_HANDLER
		except AttributeError, err:
			if not formathandler.FormatHandler.TYPE_REGISTRY:
				formathandler.FormatHandler.loadAll()
			return formathandler.FormatHandler.chooseOutput( )
	returnHandler = classmethod( logs.logOnFail( returnHandler, log ) )
	def zeros( cls, dims, typeCode=None ):
		"""Allocate a return array of the given dimensions filled with zeros"""
		return cls.returnHandler().zeros( dims, typeCode or cls.typeConstant )
	zeros = classmethod( logs.logOnFail( zeros, log ) )
	def dimensions( cls, value ):
		"""Given a data-value, get the dimensions (assumes full structure info)"""
		return cls.getHandler(value).dimensions( value )
	dimensions = classmethod( logs.logOnFail( dimensions, log ) )
	
	def arrayByteCount( cls, value ):
		"""Given a data-value, try to determine number of bytes it's final form occupies
		
		For most data-types this is arraySize() * atomic-unit-size
		"""
		return cls.getHandler(value).arrayByteCount( value )
	arrayByteCount = classmethod( logs.logOnFail( arrayByteCount, log ) )
		

# the final array data-type classes...
class GLclampdArray( ArrayDatatype, ctypes.POINTER(constants.GLclampd )):
	"""Array datatype for GLclampd types"""
	baseType = constants.GLclampd
	typeConstant = constants.GL_DOUBLE

class GLclampfArray( ArrayDatatype, ctypes.POINTER(constants.GLclampf )):
	"""Array datatype for GLclampf types"""
	baseType = constants.GLclampf
	typeConstant = constants.GL_FLOAT

class GLfloatArray( ArrayDatatype, ctypes.POINTER(constants.GLfloat )):
	"""Array datatype for GLfloat types"""
	baseType = constants.GLfloat
	typeConstant = constants.GL_FLOAT

class GLdoubleArray( ArrayDatatype, ctypes.POINTER(constants.GLdouble )):
	"""Array datatype for GLdouble types"""
	baseType = constants.GLdouble
	typeConstant = constants.GL_DOUBLE

class GLbyteArray( ArrayDatatype, ctypes.POINTER(constants.GLbyte )):
	"""Array datatype for GLbyte types"""
	baseType = constants.GLbyte
	typeConstant = constants.GL_BYTE

class GLcharArray( ArrayDatatype, ctypes.c_char_p):
	"""Array datatype for ARB extension pointers-to-arrays"""
	baseType = constants.GLchar
	typeConstant = constants.GL_BYTE
GLcharARBArray = GLcharArray

class GLshortArray( ArrayDatatype, ctypes.POINTER(constants.GLshort )):
	"""Array datatype for GLshort types"""
	baseType = constants.GLshort
	typeConstant = constants.GL_SHORT

class GLintArray( ArrayDatatype, ctypes.POINTER(constants.GLint )):
	"""Array datatype for GLint types"""
	baseType = constants.GLint
	typeConstant = constants.GL_INT

class GLubyteArray( ArrayDatatype, ctypes.POINTER(constants.GLubyte )):
	"""Array datatype for GLubyte types"""
	baseType = constants.GLubyte
	typeConstant = constants.GL_UNSIGNED_BYTE
GLbooleanArray = GLubyteArray

class GLushortArray( ArrayDatatype, ctypes.POINTER(constants.GLushort )):
	"""Array datatype for GLushort types"""
	baseType = constants.GLushort
	typeConstant = constants.GL_UNSIGNED_SHORT

class GLuintArray( ArrayDatatype, ctypes.POINTER(constants.GLuint )):
	"""Array datatype for GLuint types"""
	baseType = constants.GLuint
	typeConstant = constants.GL_UNSIGNED_INT

class GLenumArray( ArrayDatatype, ctypes.POINTER(constants.GLenum )):
	"""Array datatype for GLenum types"""
	baseType = constants.GLenum
	typeConstant = constants.GL_UNSIGNED_INT
class GLsizeiArray( ArrayDatatype, ctypes.POINTER(constants.GLsizei )):
	"""Array datatype for GLenum types"""
	baseType = constants.GLsizei
	typeConstant = constants.GL_INT

GL_CONSTANT_TO_ARRAY_TYPE = {
	constants.GL_DOUBLE : GLclampdArray,
	constants.GL_FLOAT : GLclampfArray,
	constants.GL_FLOAT : GLfloatArray,
	constants.GL_DOUBLE : GLdoubleArray,
	constants.GL_BYTE : GLbyteArray,
	constants.GL_SHORT : GLshortArray,
	constants.GL_INT : GLintArray,
	constants.GL_UNSIGNED_BYTE : GLubyteArray,
	constants.GL_UNSIGNED_SHORT : GLushortArray,
	constants.GL_UNSIGNED_INT : GLuintArray,
	#constants.GL_UNSIGNED_INT : GLenumArray,
}



'''OpenGL extension ARB.fragment_program

This module customises the behaviour of the 
OpenGL.raw.GL.ARB.fragment_program to provide a more 
Python-friendly API
'''
from OpenGL import platform, constants, constant, arrays
from OpenGL import extensions, wrapper
from OpenGL.GL import glget
import ctypes
from OpenGL.raw.GL.ARB.fragment_program import *
### END AUTOGENERATED SECTION
from OpenGL.GL import glget
glget.addGLGetConstant( GL_FRAGMENT_PROGRAM_ARB, (1,) )



'''OpenGL extension ARB.shader_objects

Overview (from the spec)
	
	This extension adds API calls that are necessary to manage shader
	objects and program objects as defined in the OpenGL 2.0 white papers by
	3Dlabs.
	
	The generation of an executable that runs on one of OpenGL's
	programmable units is modeled to that of developing a typical C/C++
	application. There are one or more source files, each of which are
	stored by OpenGL in a shader object. Each shader object (source file)
	needs to be compiled and attached to a program object. Once all shader
	objects are compiled successfully, the program object needs to be linked
	to produce an executable. This executable is part of the program object,
	and can now be loaded onto the programmable units to make it part of the
	current OpenGL state. Both the compile and link stages generate a text
	string that can be queried to get more information. This information
	could be, but is not limited to, compile errors, link errors,
	optimization hints, etc. Values for uniform variables, declared in a
	shader, can be set by the application and used to control a shader's
	behavior.
	
	This extension defines functions for creating shader objects and program
	objects, for compiling shader objects, for linking program objects, for
	attaching shader objects to program objects, and for using a program
	object as part of current state. Functions to load uniform values are
	also defined. Some house keeping functions, like deleting an object and
	querying object state, are also provided.
	
	Although this extension defines the API for creating shader objects, it
	does not define any specific types of shader objects. It is assumed that
	this extension will be implemented along with at least one such
	additional extension for creating a specific type of OpenGL 2.0 shader
	(e.g., the ARB_fragment_shader extension or the ARB_vertex_shader
	extension).

The official definition of this extension is available here:
	http://oss.sgi.com/projects/ogl-sample/registry/ARB/shader_objects.txt

Automatically generated by the get_gl_extensions script, do not edit!
'''
from OpenGL import platform, constants, constant, arrays
from OpenGL import extensions
from OpenGL.GL import glget
import ctypes
EXTENSION_NAME = 'GL_ARB_shader_objects'
GL_PROGRAM_OBJECT_ARB = constant.Constant( 'GL_PROGRAM_OBJECT_ARB', 0x8B40 )
GL_SHADER_OBJECT_ARB = constant.Constant( 'GL_SHADER_OBJECT_ARB', 0x8B48 )
GL_OBJECT_TYPE_ARB = constant.Constant( 'GL_OBJECT_TYPE_ARB', 0x8B4E )
GL_OBJECT_SUBTYPE_ARB = constant.Constant( 'GL_OBJECT_SUBTYPE_ARB', 0x8B4F )
GL_FLOAT_VEC2_ARB = constant.Constant( 'GL_FLOAT_VEC2_ARB', 0x8B50 )
GL_FLOAT_VEC3_ARB = constant.Constant( 'GL_FLOAT_VEC3_ARB', 0x8B51 )
GL_FLOAT_VEC4_ARB = constant.Constant( 'GL_FLOAT_VEC4_ARB', 0x8B52 )
GL_INT_VEC2_ARB = constant.Constant( 'GL_INT_VEC2_ARB', 0x8B53 )
GL_INT_VEC3_ARB = constant.Constant( 'GL_INT_VEC3_ARB', 0x8B54 )
GL_INT_VEC4_ARB = constant.Constant( 'GL_INT_VEC4_ARB', 0x8B55 )
GL_BOOL_ARB = constant.Constant( 'GL_BOOL_ARB', 0x8B56 )
GL_BOOL_VEC2_ARB = constant.Constant( 'GL_BOOL_VEC2_ARB', 0x8B57 )
GL_BOOL_VEC3_ARB = constant.Constant( 'GL_BOOL_VEC3_ARB', 0x8B58 )
GL_BOOL_VEC4_ARB = constant.Constant( 'GL_BOOL_VEC4_ARB', 0x8B59 )
GL_FLOAT_MAT2_ARB = constant.Constant( 'GL_FLOAT_MAT2_ARB', 0x8B5A )
GL_FLOAT_MAT3_ARB = constant.Constant( 'GL_FLOAT_MAT3_ARB', 0x8B5B )
GL_FLOAT_MAT4_ARB = constant.Constant( 'GL_FLOAT_MAT4_ARB', 0x8B5C )
GL_SAMPLER_1D_ARB = constant.Constant( 'GL_SAMPLER_1D_ARB', 0x8B5D )
GL_SAMPLER_2D_ARB = constant.Constant( 'GL_SAMPLER_2D_ARB', 0x8B5E )
GL_SAMPLER_3D_ARB = constant.Constant( 'GL_SAMPLER_3D_ARB', 0x8B5F )
GL_SAMPLER_CUBE_ARB = constant.Constant( 'GL_SAMPLER_CUBE_ARB', 0x8B60 )
GL_SAMPLER_1D_SHADOW_ARB = constant.Constant( 'GL_SAMPLER_1D_SHADOW_ARB', 0x8B61 )
GL_SAMPLER_2D_SHADOW_ARB = constant.Constant( 'GL_SAMPLER_2D_SHADOW_ARB', 0x8B62 )
GL_SAMPLER_2D_RECT_ARB = constant.Constant( 'GL_SAMPLER_2D_RECT_ARB', 0x8B63 )
GL_SAMPLER_2D_RECT_SHADOW_ARB = constant.Constant( 'GL_SAMPLER_2D_RECT_SHADOW_ARB', 0x8B64 )
GL_OBJECT_DELETE_STATUS_ARB = constant.Constant( 'GL_OBJECT_DELETE_STATUS_ARB', 0x8B80 )
GL_OBJECT_COMPILE_STATUS_ARB = constant.Constant( 'GL_OBJECT_COMPILE_STATUS_ARB', 0x8B81 )
GL_OBJECT_LINK_STATUS_ARB = constant.Constant( 'GL_OBJECT_LINK_STATUS_ARB', 0x8B82 )
GL_OBJECT_VALIDATE_STATUS_ARB = constant.Constant( 'GL_OBJECT_VALIDATE_STATUS_ARB', 0x8B83 )
GL_OBJECT_INFO_LOG_LENGTH_ARB = constant.Constant( 'GL_OBJECT_INFO_LOG_LENGTH_ARB', 0x8B84 )
GL_OBJECT_ATTACHED_OBJECTS_ARB = constant.Constant( 'GL_OBJECT_ATTACHED_OBJECTS_ARB', 0x8B85 )
GL_OBJECT_ACTIVE_UNIFORMS_ARB = constant.Constant( 'GL_OBJECT_ACTIVE_UNIFORMS_ARB', 0x8B86 )
GL_OBJECT_ACTIVE_UNIFORM_MAX_LENGTH_ARB = constant.Constant( 'GL_OBJECT_ACTIVE_UNIFORM_MAX_LENGTH_ARB', 0x8B87 )
GL_OBJECT_SHADER_SOURCE_LENGTH_ARB = constant.Constant( 'GL_OBJECT_SHADER_SOURCE_LENGTH_ARB', 0x8B88 )
glDeleteObjectARB = platform.createExtensionFunction( 
	'glDeleteObjectARB', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLhandleARB,),
	doc = 'glDeleteObjectARB( GLhandleARB(obj) ) -> None',
	argNames = ('obj',),
)

glGetHandleARB = platform.createExtensionFunction( 
	'glGetHandleARB', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=constants.GLhandleARB, 
	argTypes=(constants.GLenum,),
	doc = 'glGetHandleARB( GLenum(pname) ) -> constants.GLhandleARB',
	argNames = ('pname',),
)

glDetachObjectARB = platform.createExtensionFunction( 
	'glDetachObjectARB', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLhandleARB, constants.GLhandleARB,),
	doc = 'glDetachObjectARB( GLhandleARB(containerObj), GLhandleARB(attachedObj) ) -> None',
	argNames = ('containerObj', 'attachedObj',),
)

glCreateShaderObjectARB = platform.createExtensionFunction( 
	'glCreateShaderObjectARB', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=constants.GLhandleARB, 
	argTypes=(constants.GLenum,),
	doc = 'glCreateShaderObjectARB( GLenum(shaderType) ) -> constants.GLhandleARB',
	argNames = ('shaderType',),
)

glShaderSourceARB = platform.createExtensionFunction( 
	'glShaderSourceARB', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLhandleARB, constants.GLsizei, ctypes.POINTER(arrays.GLcharARBArray), arrays.GLintArray,),
	doc = 'glShaderSourceARB( GLhandleARB(shaderObj), GLsizei(count), POINTER(arrays.GLcharARBArray)(string), GLintArray(length) ) -> None',
	argNames = ('shaderObj', 'count', 'string', 'length',),
)

glCompileShaderARB = platform.createExtensionFunction( 
	'glCompileShaderARB', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLhandleARB,),
	doc = 'glCompileShaderARB( GLhandleARB(shaderObj) ) -> None',
	argNames = ('shaderObj',),
)

glCreateProgramObjectARB = platform.createExtensionFunction( 
	'glCreateProgramObjectARB', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=constants.GLhandleARB, 
	argTypes=(),
	doc = 'glCreateProgramObjectARB(  ) -> constants.GLhandleARB',
	argNames = (),
)

glAttachObjectARB = platform.createExtensionFunction( 
	'glAttachObjectARB', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLhandleARB, constants.GLhandleARB,),
	doc = 'glAttachObjectARB( GLhandleARB(containerObj), GLhandleARB(obj) ) -> None',
	argNames = ('containerObj', 'obj',),
)

glLinkProgramARB = platform.createExtensionFunction( 
	'glLinkProgramARB', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLhandleARB,),
	doc = 'glLinkProgramARB( GLhandleARB(programObj) ) -> None',
	argNames = ('programObj',),
)

glUseProgramObjectARB = platform.createExtensionFunction( 
	'glUseProgramObjectARB', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLhandleARB,),
	doc = 'glUseProgramObjectARB( GLhandleARB(programObj) ) -> None',
	argNames = ('programObj',),
)

glValidateProgramARB = platform.createExtensionFunction( 
	'glValidateProgramARB', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLhandleARB,),
	doc = 'glValidateProgramARB( GLhandleARB(programObj) ) -> None',
	argNames = ('programObj',),
)

glUniform1fARB = platform.createExtensionFunction( 
	'glUniform1fARB', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLint, constants.GLfloat,),
	doc = 'glUniform1fARB( GLint(location), GLfloat(v0) ) -> None',
	argNames = ('location', 'v0',),
)

glUniform2fARB = platform.createExtensionFunction( 
	'glUniform2fARB', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLint, constants.GLfloat, constants.GLfloat,),
	doc = 'glUniform2fARB( GLint(location), GLfloat(v0), GLfloat(v1) ) -> None',
	argNames = ('location', 'v0', 'v1',),
)

glUniform3fARB = platform.createExtensionFunction( 
	'glUniform3fARB', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLint, constants.GLfloat, constants.GLfloat, constants.GLfloat,),
	doc = 'glUniform3fARB( GLint(location), GLfloat(v0), GLfloat(v1), GLfloat(v2) ) -> None',
	argNames = ('location', 'v0', 'v1', 'v2',),
)

glUniform4fARB = platform.createExtensionFunction( 
	'glUniform4fARB', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLint, constants.GLfloat, constants.GLfloat, constants.GLfloat, constants.GLfloat,),
	doc = 'glUniform4fARB( GLint(location), GLfloat(v0), GLfloat(v1), GLfloat(v2), GLfloat(v3) ) -> None',
	argNames = ('location', 'v0', 'v1', 'v2', 'v3',),
)

glUniform1iARB = platform.createExtensionFunction( 
	'glUniform1iARB', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLint, constants.GLint,),
	doc = 'glUniform1iARB( GLint(location), GLint(v0) ) -> None',
	argNames = ('location', 'v0',),
)

glUniform2iARB = platform.createExtensionFunction( 
	'glUniform2iARB', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLint, constants.GLint, constants.GLint,),
	doc = 'glUniform2iARB( GLint(location), GLint(v0), GLint(v1) ) -> None',
	argNames = ('location', 'v0', 'v1',),
)

glUniform3iARB = platform.createExtensionFunction( 
	'glUniform3iARB', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLint, constants.GLint, constants.GLint, constants.GLint,),
	doc = 'glUniform3iARB( GLint(location), GLint(v0), GLint(v1), GLint(v2) ) -> None',
	argNames = ('location', 'v0', 'v1', 'v2',),
)

glUniform4iARB = platform.createExtensionFunction( 
	'glUniform4iARB', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLint, constants.GLint, constants.GLint, constants.GLint, constants.GLint,),
	doc = 'glUniform4iARB( GLint(location), GLint(v0), GLint(v1), GLint(v2), GLint(v3) ) -> None',
	argNames = ('location', 'v0', 'v1', 'v2', 'v3',),
)

glUniform1fvARB = platform.createExtensionFunction( 
	'glUniform1fvARB', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLint, constants.GLsizei, arrays.GLfloatArray,),
	doc = 'glUniform1fvARB( GLint(location), GLsizei(count), GLfloatArray(value) ) -> None',
	argNames = ('location', 'count', 'value',),
)

glUniform2fvARB = platform.createExtensionFunction( 
	'glUniform2fvARB', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLint, constants.GLsizei, arrays.GLfloatArray,),
	doc = 'glUniform2fvARB( GLint(location), GLsizei(count), GLfloatArray(value) ) -> None',
	argNames = ('location', 'count', 'value',),
)

glUniform3fvARB = platform.createExtensionFunction( 
	'glUniform3fvARB', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLint, constants.GLsizei, arrays.GLfloatArray,),
	doc = 'glUniform3fvARB( GLint(location), GLsizei(count), GLfloatArray(value) ) -> None',
	argNames = ('location', 'count', 'value',),
)

glUniform4fvARB = platform.createExtensionFunction( 
	'glUniform4fvARB', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLint, constants.GLsizei, arrays.GLfloatArray,),
	doc = 'glUniform4fvARB( GLint(location), GLsizei(count), GLfloatArray(value) ) -> None',
	argNames = ('location', 'count', 'value',),
)

glUniform1ivARB = platform.createExtensionFunction( 
	'glUniform1ivARB', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLint, constants.GLsizei, arrays.GLintArray,),
	doc = 'glUniform1ivARB( GLint(location), GLsizei(count), GLintArray(value) ) -> None',
	argNames = ('location', 'count', 'value',),
)

glUniform2ivARB = platform.createExtensionFunction( 
	'glUniform2ivARB', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLint, constants.GLsizei, arrays.GLintArray,),
	doc = 'glUniform2ivARB( GLint(location), GLsizei(count), GLintArray(value) ) -> None',
	argNames = ('location', 'count', 'value',),
)

glUniform3ivARB = platform.createExtensionFunction( 
	'glUniform3ivARB', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLint, constants.GLsizei, arrays.GLintArray,),
	doc = 'glUniform3ivARB( GLint(location), GLsizei(count), GLintArray(value) ) -> None',
	argNames = ('location', 'count', 'value',),
)

glUniform4ivARB = platform.createExtensionFunction( 
	'glUniform4ivARB', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLint, constants.GLsizei, arrays.GLintArray,),
	doc = 'glUniform4ivARB( GLint(location), GLsizei(count), GLintArray(value) ) -> None',
	argNames = ('location', 'count', 'value',),
)

glUniformMatrix2fvARB = platform.createExtensionFunction( 
	'glUniformMatrix2fvARB', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLint, constants.GLsizei, constants.GLboolean, arrays.GLfloatArray,),
	doc = 'glUniformMatrix2fvARB( GLint(location), GLsizei(count), GLboolean(transpose), GLfloatArray(value) ) -> None',
	argNames = ('location', 'count', 'transpose', 'value',),
)

glUniformMatrix3fvARB = platform.createExtensionFunction( 
	'glUniformMatrix3fvARB', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLint, constants.GLsizei, constants.GLboolean, arrays.GLfloatArray,),
	doc = 'glUniformMatrix3fvARB( GLint(location), GLsizei(count), GLboolean(transpose), GLfloatArray(value) ) -> None',
	argNames = ('location', 'count', 'transpose', 'value',),
)

glUniformMatrix4fvARB = platform.createExtensionFunction( 
	'glUniformMatrix4fvARB', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLint, constants.GLsizei, constants.GLboolean, arrays.GLfloatArray,),
	doc = 'glUniformMatrix4fvARB( GLint(location), GLsizei(count), GLboolean(transpose), GLfloatArray(value) ) -> None',
	argNames = ('location', 'count', 'transpose', 'value',),
)

glGetObjectParameterfvARB = platform.createExtensionFunction( 
	'glGetObjectParameterfvARB', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLhandleARB, constants.GLenum, arrays.GLfloatArray,),
	doc = 'glGetObjectParameterfvARB( GLhandleARB(obj), GLenum(pname), GLfloatArray(params) ) -> None',
	argNames = ('obj', 'pname', 'params',),
)

glGetObjectParameterivARB = platform.createExtensionFunction( 
	'glGetObjectParameterivARB', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLhandleARB, constants.GLenum, arrays.GLintArray,),
	doc = 'glGetObjectParameterivARB( GLhandleARB(obj), GLenum(pname), GLintArray(params) ) -> None',
	argNames = ('obj', 'pname', 'params',),
)

glGetInfoLogARB = platform.createExtensionFunction( 
	'glGetInfoLogARB', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLhandleARB, constants.GLsizei, arrays.GLsizeiArray, arrays.GLcharARBArray,),
	doc = 'glGetInfoLogARB( GLhandleARB(obj), GLsizei(maxLength), GLsizeiArray(length), GLcharARBArray(infoLog) ) -> None',
	argNames = ('obj', 'maxLength', 'length', 'infoLog',),
)

glGetAttachedObjectsARB = platform.createExtensionFunction( 
	'glGetAttachedObjectsARB', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLhandleARB, constants.GLsizei, arrays.GLsizeiArray, arrays.GLuintArray,),
	doc = 'glGetAttachedObjectsARB( GLhandleARB(containerObj), GLsizei(maxCount), GLsizeiArray(count), GLuintArray(obj) ) -> None',
	argNames = ('containerObj', 'maxCount', 'count', 'obj',),
)

glGetUniformLocationARB = platform.createExtensionFunction( 
	'glGetUniformLocationARB', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=constants.GLint, 
	argTypes=(constants.GLhandleARB, arrays.GLcharARBArray,),
	doc = 'glGetUniformLocationARB( GLhandleARB(programObj), GLcharARBArray(name) ) -> constants.GLint',
	argNames = ('programObj', 'name',),
)

glGetActiveUniformARB = platform.createExtensionFunction( 
	'glGetActiveUniformARB', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLhandleARB, constants.GLuint, constants.GLsizei, arrays.GLsizeiArray, arrays.GLintArray, arrays.GLuintArray, arrays.GLcharARBArray,),
	doc = 'glGetActiveUniformARB( GLhandleARB(programObj), GLuint(index), GLsizei(maxLength), GLsizeiArray(length), GLintArray(size), GLuintArray(type), GLcharARBArray(name) ) -> None',
	argNames = ('programObj', 'index', 'maxLength', 'length', 'size', 'type', 'name',),
)

glGetUniformfvARB = platform.createExtensionFunction( 
	'glGetUniformfvARB', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLhandleARB, constants.GLint, arrays.GLfloatArray,),
	doc = 'glGetUniformfvARB( GLhandleARB(programObj), GLint(location), GLfloatArray(params) ) -> None',
	argNames = ('programObj', 'location', 'params',),
)

glGetUniformivARB = platform.createExtensionFunction( 
	'glGetUniformivARB', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLhandleARB, constants.GLint, arrays.GLintArray,),
	doc = 'glGetUniformivARB( GLhandleARB(programObj), GLint(location), GLintArray(params) ) -> None',
	argNames = ('programObj', 'location', 'params',),
)

glGetShaderSourceARB = platform.createExtensionFunction( 
	'glGetShaderSourceARB', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLhandleARB, constants.GLsizei, arrays.GLsizeiArray, arrays.GLcharARBArray,),
	doc = 'glGetShaderSourceARB( GLhandleARB(obj), GLsizei(maxLength), GLsizeiArray(length), GLcharARBArray(source) ) -> None',
	argNames = ('obj', 'maxLength', 'length', 'source',),
)


def glInitShaderObjectsARB():
	'''Return boolean indicating whether this extension is available'''
	return extensions.hasGLExtension( EXTENSION_NAME )



#!/usr/bin/env python

# This is statement is required by the build system to query build info
if __name__ == '__build__':
	raise Exception

# A class that creates an opengl widget.
# Mike Hartshorn
# Department of Chemistry
# University of York, UK
# 

from OpenGL.GL import *
from OpenGL.GLU import *
from Tkinter import _default_root
from Tkinter import *
import math
import os,sys

#

def glTranslateScene(s, x, y, mousex, mousey):
	glMatrixMode(GL_MODELVIEW)
	mat = glGetDoublev(GL_MODELVIEW_MATRIX)
	glLoadIdentity()
	glTranslatef(s * (x - mousex), s * (mousey - y), 0.0)
	glMultMatrixd(mat)


def glRotateScene(s, xcenter, ycenter, zcenter, x, y, mousex, mousey):
	glMatrixMode(GL_MODELVIEW)
	mat = glGetDoublev(GL_MODELVIEW_MATRIX)
	glLoadIdentity()
	glTranslatef(xcenter, ycenter, zcenter)
	glRotatef(s * (y - mousey), 1., 0., 0.)
	glRotatef(s * (x - mousex), 0., 1., 0.)
	glTranslatef(-xcenter, -ycenter, -zcenter)
	glMultMatrixd(mat)


def sub(x, y):
	return map(lambda a, b: a-b, x, y)


def dot(x, y):
	t = 0
	for i in range(len(x)):
		t = t + x[i]*y[i]
	return t


def glDistFromLine(x, p1, p2):
	f = map(lambda x, y: x-y, p2, p1)
	g = map(lambda x, y: x-y, x, p1)
	return dot(g, g) - dot(f, g)**2/dot(f, f)


# Keith Junius <junius@chem.rug.nl> provided many changes to Togl
TOGL_NORMAL = 1
TOGL_OVERLAY = 2

def v3distsq(a,b):
	d = ( a[0] - b[0], a[1] - b[1], a[2] - b[2] )
	return d[0]*d[0] + d[1]*d[1] + d[2]*d[2]

# new version from Daniel Faken (Daniel_Faken@brown.edu) for static 
# loading comptability
if _default_root is None:
	_default_root = Tk()

# add this file's directory to Tcl's search path
# on Linux Togl is installed in ./linux2-tk8.0
# on Windows (Python2.0) in ./win32-tk8.3  
try:
	TOGL_DLL_PATH = os.path.join(
		os.path.dirname(__file__),
		sys.platform + "-tk" + _default_root.getvar("tk_version")
	)
except NameError, err:
	# no __file__, likely running as an egg
	TOGL_DLL_PATH = ""
_default_root.tk.call('lappend', 'auto_path', TOGL_DLL_PATH)
try:
	_default_root.tk.eval('load {} Togl')
except TclError:
	pass
_default_root.tk.call('package', 'require', 'Togl')

# This code is needed to avoid faults on sys.exit()
# [DAA, Jan 1998]
import sys
oldexitfunc = None
if hasattr(sys, 'exitfunc'):
	oldexitfunc = sys.exitfunc
def cleanup():
	from Tkinter import _default_root, TclError
	import Tkinter
	try: 
		if _default_root: _default_root.destroy()
	except TclError:
		pass
	Tkinter._default_root = None
	if oldexitfunc: oldexitfunc()
sys.exitfunc = cleanup
# [end DAA]

class Togl(Widget):
	"""
	Togl Widget
	Keith Junius
	Department of Biophysical Chemistry
	University of Groningen, The Netherlands
	Very basic widget which provides access to Togl functions.
	N.B. this requires a modified version of Togl 1.5 to gain access to the
	extra functionality. This support should be included in Togl 1.6, I hope.
	"""


	def __init__(self, master=None, cnf={}, **kw):
		Widget.__init__(self, master, 'togl', cnf, kw)


	def render(self):
		return


	def swapbuffers(self):
		self.tk.call(self._w, 'swapbuffers')


	def makecurrent(self):
		self.tk.call(self._w, 'makecurrent')


	def alloccolor(self, red, green, blue):
		return self.tk.getint(self.tk.call(self._w, 'alloccolor', red, green, blue))


	def freecolor(self, index):
		self.tk.call(self._w, 'freecolor', index)


	def setcolor(self, index, red, green, blue):
		self.tk.call(self._w, 'setcolor', index, red, green, blue)


	def loadbitmapfont(self, fontname):
		return self.tk.getint(self.tk.call(self._w, 'loadbitmapfont', fontname))


	def unloadbitmapfont(self, fontbase):
		self.tk.call(self._w, 'unloadbitmapfont', fontbase)


	def uselayer(self, layer):
		self.tk.call(self._w, 'uselayer', layer)


	def showoverlay(self):
		self.tk.call(self._w, 'showoverlay')


	def hideoverlay(self):
		self.tk.call(self._w, 'hideoverlay')


	def existsoverlay(self):
		return self.tk.getboolean(self.tk.call(self._w, 'existsoverlay'))


	def getoverlaytransparentvalue(self):
		return self.tk.getint(self.tk.call(self._w, 'getoverlaytransparentvalue'))


	def ismappedoverlay(self):
		return self.tk.getboolean(self.tk.call(self._w, 'ismappedoverlay'))


	def alloccoloroverlay(self, red, green, blue):
		return self.tk.getint(self.tk.call(self._w, 'alloccoloroverlay', red, green, blue))


	def freecoloroverlay(self, index):
		self.tk.call(self._w, 'freecoloroverlay', index)



class RawOpengl(Widget, Misc):
	"""Widget without any sophisticated bindings\
	by Tom Schwaller"""


	def __init__(self, master=None, cnf={}, **kw):
		Widget.__init__(self, master, 'togl', cnf, kw)
		self.bind('<Map>', self.tkMap)
		self.bind('<Expose>', self.tkExpose)
		self.bind('<Configure>', self.tkExpose)


	def tkRedraw(self, *dummy):
		# This must be outside of a pushmatrix, since a resize event
		# will call redraw recursively. 
		self.update_idletasks()
		self.tk.call(self._w, 'makecurrent')
		_mode = glGetDoublev(GL_MATRIX_MODE)
		try:
			glMatrixMode(GL_PROJECTION)
			glPushMatrix()
			try:
				self.redraw()
				glFlush()
			finally:
				glPopMatrix()
		finally:
			glMatrixMode(_mode)
		self.tk.call(self._w, 'swapbuffers')


	def tkMap(self, *dummy):
		self.tkExpose()


	def tkExpose(self, *dummy):
		self.tkRedraw()




class Opengl(RawOpengl):
	"""\
Tkinter bindings for an Opengl widget.
Mike Hartshorn
Department of Chemistry
University of York, UK
http://www.yorvic.york.ac.uk/~mjh/
"""

	def __init__(self, master=None, cnf={}, **kw):
		"""\
		Create an opengl widget.
		Arrange for redraws when the window is exposed or when
		it changes size."""

		#Widget.__init__(self, master, 'togl', cnf, kw)
		apply(RawOpengl.__init__, (self, master, cnf), kw)
		self.initialised = 0

		# Current coordinates of the mouse.
		self.xmouse = 0
		self.ymouse = 0

		# Where we are centering.
		self.xcenter = 0.0
		self.ycenter = 0.0
		self.zcenter = 0.0

		# The _back color
		self.r_back = 1.
		self.g_back = 0.
		self.b_back = 1.

		# Where the eye is
		self.distance = 10.0

		# Field of view in y direction
		self.fovy = 30.0

		# Position of clipping planes.
		self.near = 0.1
		self.far = 1000.0

		# Is the widget allowed to autospin?
		self.autospin_allowed = 0

		# Is the widget currently autospinning?
		self.autospin = 0

		# Basic bindings for the virtual trackball
		self.bind('<Map>', self.tkMap)
		self.bind('<Expose>', self.tkExpose)
		self.bind('<Configure>', self.tkExpose)
		self.bind('<Shift-Button-1>', self.tkHandlePick)
		#self.bind('<Button-1><ButtonRelease-1>', self.tkHandlePick)
		self.bind('<Button-1>', self.tkRecordMouse)
		self.bind('<B1-Motion>', self.tkTranslate)
		self.bind('<Button-2>', self.StartRotate)
		self.bind('<B2-Motion>', self.tkRotate)
		self.bind('<ButtonRelease-2>', self.tkAutoSpin)
		self.bind('<Button-3>', self.tkRecordMouse)
		self.bind('<B3-Motion>', self.tkScale)


	def help(self):
		"""Help for the widget."""

		import Dialog
		d = Dialog.Dialog(None, {'title': 'Viewer help',
		                         'text': 'Button-1: Translate\n'
		                                 'Button-2: Rotate\n'
		                                 'Button-3: Zoom\n'
		                                 'Reset: Resets transformation to identity\n',
		                         'bitmap': 'questhead',
		                         'default': 0,
		                         'strings': ('Done', 'Ok')})


	def activate(self):
		"""Cause this Opengl widget to be the current destination for drawing."""

		self.tk.call(self._w, 'makecurrent')


	# This should almost certainly be part of some derived class.
	# But I have put it here for convenience.
	def basic_lighting(self):
		"""\
		Set up some basic lighting (single infinite light source).

		Also switch on the depth buffer."""
   
		self.activate()
		light_position = (1, 1, 1, 0)
		glLightfv(GL_LIGHT0, GL_POSITION, light_position)
		glEnable(GL_LIGHTING)
		glEnable(GL_LIGHT0)
		glDepthFunc(GL_LESS)
		glEnable(GL_DEPTH_TEST)
		glMatrixMode(GL_MODELVIEW)
		glLoadIdentity()


	def report_opengl_errors(message = "OpenGL error:"):
		"""Report any opengl errors that occured while drawing."""

		print 'report_opengl_errors is now useless.  glGetError replaced by GLexception'
#		while 1:
#			err_value = glGetError()
#			if not err_value: break	 
#			print message, gluErrorString(err_value)


	def set_background(self, r, g, b):
		"""Change the background colour of the widget."""

		self.r_back = r
		self.g_back = g
		self.b_back = b

		self.tkRedraw()


	def set_centerpoint(self, x, y, z):
		"""Set the new center point for the model.
		This is where we are looking."""

		self.xcenter = x
		self.ycenter = y
		self.zcenter = z

		self.tkRedraw()


	def set_eyepoint(self, distance):
		"""Set how far the eye is from the position we are looking."""

		self.distance = distance
		self.tkRedraw()


	def reset(self):
		"""Reset rotation matrix for this widget."""

		glMatrixMode(GL_MODELVIEW)
		glLoadIdentity()
		self.tkRedraw()


	def tkHandlePick(self, event):
		"""Handle a pick on the scene."""

		if hasattr(self, 'pick'):
			# here we need to use glu.UnProject

			# Tk and X have their origin top left, 
			# while Opengl has its origin bottom left.
			# So we need to subtract y from the window height to get
			# the proper pick position for Opengl

			realy = self.winfo_height() - event.y

			p1 = gluUnProject(event.x, realy, 0.)
			p2 = gluUnProject(event.x, realy, 1.)

			if self.pick(self, p1, p2):
				"""If the pick method returns true we redraw the scene."""

				self.tkRedraw()


	def tkRecordMouse(self, event):
		"""Record the current mouse position."""

		self.xmouse = event.x
		self.ymouse = event.y


	def StartRotate(self, event):
		# Switch off any autospinning if it was happening

		self.autospin = 0
		self.tkRecordMouse(event)


	def tkScale(self, event):
		"""Scale the scene.  Achieved by moving the eye position.

		Dragging up zooms in, while dragging down zooms out
		"""
		scale = 1 - 0.01 * (event.y - self.ymouse)
		# do some sanity checks, scale no more than
		# 1:1000 on any given click+drag
		if scale < 0.001:
			scale = 0.001
		elif scale > 1000:
			scale = 1000
		self.distance = self.distance * scale
		self.tkRedraw()
		self.tkRecordMouse(event)


	def do_AutoSpin(self):
		s = 0.5
		self.activate()

		glRotateScene(0.5, self.xcenter, self.ycenter, self.zcenter, self.yspin, self.xspin, 0, 0)
		self.tkRedraw()

		if self.autospin:
			self.after(10, self.do_AutoSpin)


	def tkAutoSpin(self, event):
		"""Perform autospin of scene."""

		self.after(4)
		self.update_idletasks()

		# This could be done with one call to pointerxy but I'm not sure
		# it would any quicker as we would have to split up the resulting
		# string and then conv

		x = self.tk.getint(self.tk.call('winfo', 'pointerx', self._w))
		y = self.tk.getint(self.tk.call('winfo', 'pointery', self._w))

		if self.autospin_allowed:
			if x != event.x_root and y != event.y_root:
				self.autospin = 1

		self.yspin = x - event.x_root
		self.xspin = y - event.y_root

		self.after(10, self.do_AutoSpin)


	def tkRotate(self, event):
		"""Perform rotation of scene."""

		self.activate()
		glRotateScene(0.5, self.xcenter, self.ycenter, self.zcenter, event.x, event.y, self.xmouse, self.ymouse)
		self.tkRedraw()
		self.tkRecordMouse(event)


	def tkTranslate(self, event):
		"""Perform translation of scene."""

		self.activate()

		# Scale mouse translations to object viewplane so object tracks with mouse
		win_height = max( 1,self.winfo_height() )
		obj_c	  = ( self.xcenter, self.ycenter, self.zcenter )
		win		= gluProject( obj_c[0], obj_c[1], obj_c[2])
		obj		= gluUnProject( win[0], win[1] + 0.5 * win_height, win[2])
		dist	   = math.sqrt( v3distsq( obj, obj_c ) )
		scale	  = abs( dist / ( 0.5 * win_height ) )

		glTranslateScene(scale, event.x, event.y, self.xmouse, self.ymouse)
		self.tkRedraw()
		self.tkRecordMouse(event)


	def tkRedraw(self, *dummy):
		"""Cause the opengl widget to redraw itself."""

		if not self.initialised: return
		self.activate()

		glPushMatrix()			# Protect our matrix
		self.update_idletasks()
		self.activate()
		w = self.winfo_width()
		h = self.winfo_height()
		glViewport(0, 0, w, h)

		# Clear the background and depth buffer.
		glClearColor(self.r_back, self.g_back, self.b_back, 0.)
		glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT)

		glMatrixMode(GL_PROJECTION)
		glLoadIdentity()
		gluPerspective(self.fovy, float(w)/float(h), self.near, self.far)

		if 0:
			# Now translate the scene origin away from the world origin
			glMatrixMode(GL_MODELVIEW)
			mat = glGetDoublev(GL_MODELVIEW_MATRIX)
			glLoadIdentity()
			glTranslatef(-self.xcenter, -self.ycenter, -(self.zcenter+self.distance))
			glMultMatrixd(mat)
		else:
			gluLookAt(self.xcenter, self.ycenter, self.zcenter + self.distance,
			    self.xcenter, self.ycenter, self.zcenter,
			    0., 1., 0.)
			glMatrixMode(GL_MODELVIEW)
	
		# Call objects redraw method.
		self.redraw(self)
		glFlush()				# Tidy up
		glPopMatrix()			# Restore the matrix

		self.tk.call(self._w, 'swapbuffers')
	def redraw( self, *args, **named ):
		"""Prevent access errors if user doesn't set redraw fast enough"""


	def tkMap(self, *dummy):
		"""Cause the opengl widget to redraw itself."""

		self.tkExpose()


	def tkExpose(self, *dummy):
		"""Redraw the widget.
		Make it active, update tk events, call redraw procedure and
		swap the buffers.  Note: swapbuffers is clever enough to
		only swap double buffered visuals."""

		self.activate()
		if not self.initialised:
			self.basic_lighting()
			self.initialised = 1
		self.tkRedraw()


	def tkPrint(self, file):
		"""Turn the current scene into PostScript via the feedback buffer."""

		self.activate()
